var topic = "<html>\n\
<head>\n\
	<script src=\"../scripts/lib/jquery-1.11.1.min.js\" type=\"text/javascript\"/></script\n\
	<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\">\n\
	<!--script type=\"text/javascript\" src=\"..\\..\\../scripts/utils/adsk.redirect.js\"/></script>	\n\
	<script type=\"text/javascript\" src=\"..\\..\\../scripts/prettify/lang-maxscript.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\..\\../scripts/prettify/run_prettify.js\"/></script-->\n\
    <title>MAXScript Help</title>\n\
    <meta name=\"topicid\" content=\"GUID-68A5D8E3-1474-4E84-8C22-5EFA4A34BF3C\" />\n\
    </head>\n\
<body height=\"100%\">\n\
  <a name=\"top\"></a>\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2022/ENU/MAXScript-Help/styles/bootstrap.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2022/ENU/MAXScript-Help/styles/additions.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2022/ENU/MAXScript-Help/styles/font-awesome.min.css\">\n\
      <script src=\"/cloudhelp/2022/ENU/MAXScript-Help/scripts/prettify/lang-maxscript.js\"></script>\n\
\n\
  <div class=\"body_content\" id=\"body-content\">\n\
  <div class=\"head-block\"><h1>Set Controller Value Functions</h1></div><p>The following functions can be used to adjust the value of a controller using either absolute or relative values, and set an animation key or shift the whole animation curve, with the ability to commit or discard the animation curve changes after setting the value. Available in <strong>3ds Max 2017 and higher</strong>.</p>\n\
<p><strong>Functions:</strong></p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">&lt;value&gt;SetControllerValue &lt;controller&gt;target &lt;value&gt;newValue &lt;boolean&gt;setKey &lt;enum&gt;{mode:#absolute|#relative}</code></pre><p>Sets the controller specified by the first argument to the absolute value specified by the second argument when the fourth argument is <code>#absolute</code>, or increments the existing value by the second argument if the fourth argument is <code>#relative</code>.</p>\n\
<p>When the third argument is set to True and the function is called while in Auto-Key mode, or inside an Animate context, a new animation key will be created at the current time in the controller.</p>\n\
<p>When the third argument is set to False and the function is called while in Auto-Key mode, or inside an Animate context, and there is an existing key at the current slider time, the existing key will be set to the new value and the animation curve will be shifted temporarily in relation to it. Changing the current time or calling <code>RestoreControllerValue()</code> will discard the value changes.</p>\n\
<p>Calling the <code>CommitControllerValue()</code> function will commit the changes, updating all existing keys to their new values, unless the changes have already been discarded.</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">&lt;void&gt;CommitControllerValue &lt;controller&gt;target</code></pre><p>Makes the temporary changes to a controller&apos;s value applied using <code>SetControllerValue()</code> with third argument set to False permanent.</p>\n\
<p>If this function is not called, the changes will be discarded as soon as the current time is changed.</p>\n\
<p>If the <code>RestoreControllerValue()</code> function has been called after the <code>SetControllerValue()</code> and the <code>CommitControllerValue()</code> calls, the function will have no effect.</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">&lt;void&gt;RestoreControllerValue &lt;controller&gt;target</code></pre><p>Discards the temporary changes to a controller&apos;s value applied using <code>SetControllerValue()</code> with third argument set to False permanent.</p>\n\
<p>It can be used to programmatically achive a similar result to moving the time slider after a <code>SetControllerValue()</code> call, however the UI will not be updated to reset the value.</p>\n\
<p>It has no effect if the changes have been already committed using <code>CommitControllerValue()</code>, or if the third argument of <code>SetControllerValue()</code> was set to True to set an animation key.</p>\n\
<blockquote>\n\
<p>   <strong>EXAMPLE:</strong></p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">   b = box()\n\
   --&gt; $Box:Box001 @ [0.000000,0.000000,0.000000]\n\
   c = b.height.controller = newDefaultFloatController()\n\
   --&gt; Controller:Bezier_Float\n\
   setControllerValue c 100 true #absolute\n\
   --&gt; 100\n\
   c.value\n\
   --&gt; 100.0\n\
   setControllerValue c -10 true #relative\n\
   --&gt; -10\n\
   c.value\n\
   --&gt; 90.0\n\
\n\
   sliderTime = 0 --make sure the time slider is on frame 0\n\
   --&gt; 0\n\
   select b --select the box\n\
   --&gt; OK\n\
   max modify mode --show the box in the modify panel\n\
   --&gt; OK\n\
\n\
   --Try to set an animation key with value 90+50=140 on frame 50 with the third argument set to false\n\
   with animate on at time 100 setControllerValue c 50 false #relative\n\
   --&gt; 50\n\
   c.keys --there are no keys created, and the box is now 140 units tall\n\
   --&gt; #keys()\n\
   c.value\n\
   --&gt; 140\n\
\n\
   --Now let&apos;s set the third argument to true to create animation keys:\n\
   with animate on at time 100 setControllerValue c -40 true #relative\n\
   --&gt; -40\n\
\n\
   --Checking the keys, the first key has the previous controller value 140 on frame 0\n\
   --The second key has the new value of 100 at frame 100:\n\
   for k in c.keys do format &quot;% = %\\n&quot; k (k.value)\n\
   --&gt; #Bezier Float key(1 @ 0f) = 140.0\n\
   --&gt; #Bezier Float key(2 @ 100f) = 100.0\n\
\n\
   sliderTime = 100    \n\
   --Now let&apos;s try to temporarily modify the second key on frame 100 to an absolute value of 200:\n\
   with animate on at time 100 setControllerValue c 200 false #absolute\n\
   --&gt; 200\n\
   --The command panel shows Height: 200.0 on frame 50, however \n\
   --checking the keys, they still show the original values of 140.0 and 100.0:\n\
   for k in c.keys do format &quot;% = %\\n&quot; k (k.value)\n\
   --&gt; #Bezier Float key(1 @ 0f) = 140.0\n\
   --&gt; #Bezier Float key(2 @ 100f) = 100.0\n\
   sliderTime -= 1\n\
   --&gt; 49\n\
   sliderTime += 1\n\
   --&gt; 50\n\
   --At this point, the Modify Panel will show 100.0 again, as the uncommited value was reset back.\n\
\n\
   --Let&apos;s do this again, but this time we will commit after the change:\n\
   with animate on at time 100 setControllerValue c 200 false #absolute\n\
   --&gt; 200\n\
   CommitControllerValue c --commit the change\n\
   --&gt; OK\n\
   --At this point, moving the time slider will not reset the new value anymore, it is committed to 200.0.\n\
   --But in addition, the first key on frame 0 was also shifted to 240.0!\n\
   for k in c.keys do format &quot;% = %\\n&quot; k (k.value)\n\
   --&gt; #Bezier Float key(1 @ 0f) = 240.0\n\
   --&gt; #Bezier Float key(2 @ 100f) = 200.0\n\
\n\
   with animate on at time 50 setControllerValue c 42 true #absolute\n\
   --&gt; 42\n\
   sliderTime = 50\n\
   --&gt; 50\n\
   with animate on at time 50 setControllerValue c 24 false #absolute\n\
   --&gt; 24\n\
   RestoreControllerValue c --we take back the change internally, but the Modify Panel still shows 42.0\n\
   --&gt; OK\n\
   CommitControllerValue c --try to commit the change of 24.0 to the key and shift the whole animation curve\n\
   --&gt; OK\n\
   sliderTime = 49\n\
   --&gt; 49\n\
   sliderTime = 50\n\
   --&gt; 50\n\
   --Since we restored the controller value before the commit, the value at frame 50 remains 42 and is not shifted to 24.\n\
   --The same is true for all other keys:\n\
   for k in c.keys do format &quot;% = %\\n&quot; k (k.value)\n\
   --&gt; #Bezier Float key(1 @ 0f) = 240.0\n\
   --&gt; #Bezier Float key(2 @ 50f) = 42.0\n\
   --&gt; #Bezier Float key(3 @ 100f) = 200.0\n\
</code></pre></blockquote>\n\
\n\
  </div>\n\
\n\
    <div class=\"related-links\">\n\
  \n\
    <div class=\"familylinks\">\n\
      <div class=\"parentLink\">\n\
        <strong>Parent page:</strong> <a class=\"link\" href=\"../Controller-Common-Properties.html\">Controller Common Properties, Operators, and Methods</a>\n\
      </div>\n\
  \n\
    </div>\n\
  \n\
    </div>\n\
\n\
</body>\n\
</html>\n\
";