var topic = "<html>\n\
<head>\n\
	<script src=\"../scripts/lib/jquery-1.11.1.min.js\" type=\"text/javascript\"/></script\n\
	<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\">\n\
	<!--script type=\"text/javascript\" src=\"..\\../scripts/utils/adsk.redirect.js\"/></script>	\n\
	<script type=\"text/javascript\" src=\"..\\../scripts/prettify/lang-maxscript.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\../scripts/prettify/run_prettify.js\"/></script-->\n\
    <title>MAXScript Help</title>\n\
    <meta name=\"topicid\" content=\"GUID-3F9BC35F-918C-46C6-8617-7DCDDD62FCE3\" />\n\
    </head>\n\
<body height=\"100%\">\n\
  <a name=\"top\"></a>\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2022/ENU/MAXScript-Help/styles/bootstrap.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2022/ENU/MAXScript-Help/styles/additions.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2022/ENU/MAXScript-Help/styles/font-awesome.min.css\">\n\
      <script src=\"/cloudhelp/2022/ENU/MAXScript-Help/scripts/prettify/lang-maxscript.js\"></script>\n\
\n\
  <div class=\"body_content\" id=\"body-content\">\n\
  <div class=\"head-block\"><h1>Indexed Access to Animatable Properties in 3ds Max Objects</h1></div><p>As an alternative to using named property accesses, you can access in an object any animatable property that is visible as a track in Track View by indexing the <strong>3ds Max</strong> object value as though it were an array.</p>\n\
<p>This simplifies such tasks as iterating over all the animatable properties in an object and accessing sub-controllers that don&apos;t have unique names, such as within list controllers.</p>\n\
<p><strong>Properties</strong></p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">&lt;maxwrapper_object&gt;.numSubs : Integer, read-only      </code></pre><p>Yields the number of accessible sub-animatables, or tracks, in an object. This is the number of immediate <strong>children subAnims</strong>, and may include as yet invisible tracks in the track view, such as visibility tracks on a node or yet-to-be animated vertices in an FFD. This property operates on any MAXWrapper subclass.</p>\n\
<p><strong>Methods</strong></p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">getSubAnimName &lt;maxwrapper_object&gt; &lt;index&gt;&#x202F;[asString:&lt;bool&gt;]&#x202F;[localizedName:&lt;bool&gt;] </code></pre><p>Returns the name of the indexed subAnimin the supplied MAXWrapper.</p>\n\
<p>If the optional <code>asString</code> keyword is supplied and set to <code>true</code> , the name is returned as a string. If <code>false</code> or not specified, a Name Value is returned.</p>\n\
<p><img src=\"./images/GUID-21C87158-BBA7-4B63-B8F7-CB9BA17CA2CA-low.gif\" alt> <a href=\"What-s-New-in-MAXScript-in-3ds.html\">NEW</a> <strong>in 3ds Max 2022:</strong> When the <code>localizedName</code> keyword parameter is <code>true</code> (the default), the name as it appears in the UI is returned.  Otherwise the non-localized name is returned.</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">getSubAnimNames &lt;maxwrapper_object&gt;&#x202F;[localizedNames:&lt;bool&gt;] </code></pre><p>Returns an array of names in subAnim index order.</p>\n\
<p><img src=\"./images/GUID-21C87158-BBA7-4B63-B8F7-CB9BA17CA2CA-low.gif\" alt> <a href=\"What-s-New-in-MAXScript-in-3ds.html\">NEW</a> <strong>in 3ds Max 2022:</strong> When the <code>localizedNames</code> keyword parameter is <code>true</code> (the default), the names as they appear in the UI is returned.  Otherwise the non-localized names are returned.</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">getSubAnim &lt;maxwrapper_object&gt; &lt;index&gt; </code></pre><p>Returns the indexed subAnim. See the index operator below.</p>\n\
<p><strong>Operators</strong></p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">&lt;maxwrapper_object&gt;[&lt;index_number&gt;|&lt;subanim_name&gt;] </code></pre><p>Returns the subAnim with the specified index or name. You can apply the index operator, <code>[&lt;index_number&gt;]</code> , to any MAXWrapper object (nodes, modifiers, controllers, materials, etc.) to access a numbered subAnim.</p>\n\
<blockquote>\n\
<p>   <strong>FOR EXAMPLE:</strong></p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">   for i in 1 to $foo.numSubs do print $foo[i] -- iterate them\n\
   $bar.position.controller[3] -- third subAnim in a position list controller\n\
   ffd_mod[i].value -- position of i&apos;th FFD control point\n\
</code></pre></blockquote>\n\
<div class=\"note note:\"><span class=\"notetitle\">Note:</span> </div>\n\
<p>You cannot set subAnims with the index operator, this is a read-only access.</p>\n\
<p>In place of an index, you can also use a subanim name. It can be any expression that yields a string or name value. So, whereas in releases prior <strong>3ds Max 4</strong>, getting at the position subAnim in a node require you to remember index 3 for the transform subanim and 1 for its position subanim, in other words</p>\n\
<blockquote>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">   $foo[3][1] -- position within transform within node\n\
</code></pre><p>   you can use instead:</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">   $foo[#transform][#position]</code></pre></blockquote>\n\
<p>The names you can use are those seen in the track view or retrieved with the functions <code>getSubAnimName()</code> or <code>getSubAnimNames().</code></p>\n\
<h2 id=\"subanim-class\">SubAnim Class</h2><p>MAXScript defines a <code>SubAnim</code> class, instances of which provide a general representation for sub-animatables. Whenever you use the index operator on a <strong>3ds Max</strong> object, it returns a SubAnim instance.</p>\n\
<blockquote>\n\
<p>   <strong>FOR EXAMPLE:</strong></p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">   $box01[3] -&gt; SubAnim:Transform\n\
</code></pre></blockquote>\n\
<p>The third subAnim in a node is typically the transform track.</p>\n\
<p><strong>Properties</strong></p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">&lt;subAnim&gt;.name : String, read-only </code></pre><p>Returns the name of the track subAnim as shown in TrackView.</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">&lt;subAnim&gt;.parent : Node, read-only </code></pre><p>Returns the true owner of the subAnim.</p>\n\
<div class=\"note note:\"><span class=\"notetitle\">Note:</span> </div>\n\
<p>The true object owning the subAnim can be different than what was originally specified when creating the subAnim, and MAXScript automatically hides certain types of subAnims, automatically promoting the subAnims within them.</p>\n\
<blockquote>\n\
<p>   <strong>FOR EXAMPLE:</strong></p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">   s=sphere()\n\
   sa = s.baseobject[1]\n\
   --&gt; SubAnim:Radius\n\
   sa.parent\n\
   --&gt; ReferenceTarget:ParamBlock\n\
   sa.index\n\
   --&gt; 1\n\
   --The actual owner of the Radius subAnim is the sphere&apos;s parameter block.\n\
</code></pre></blockquote>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">&lt;subAnim&gt;.index : Integer, read-only </code></pre><p>Returns the index of the subAnim.</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">&lt;subAnim&gt;.value : Value </code></pre><p>The current value of the track subAnim, equivalent to <code>&lt;controller&gt;.value</code> . Returns <code>undefined</code> if no appropriate value exists or the track has not yet been assigned a controller. The <code>.value</code> property yields the value at the current MAXScript time. You can assign to this value to set the track&apos;s value at that time.</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">&lt;subAnim&gt;.controller : Controller </code></pre><p>The controller for the track SubAnim. Returns <code>undefined</code> if a controller has not yet been assigned or if the SubAnim is not animatable. You can assign a new controller to a track by assigning to the <code>.controller</code> property of its subAnim.</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">&lt;subAnim&gt;.keys : KeyArray, read-only </code></pre><p>Yields the key array for that track or <code>undefined</code> if not keyable or a controller has not yet been assigned. This property is read-only.</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">&lt;subAnim&gt;.isAnimated : Boolean, read-only </code></pre><p>Yields <code>true</code> if there is an animated controller present. This property is read-only.</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">&lt;subAnim&gt;.object : MAXWrapper, read-only </code></pre><p>Yields the subAnim object, or <code>undefined</code> if not assigned. This property is read-only. The <code>.object</code> property may return any kind of MAXWrapper object, depending on what the parent object decides to put there. For example, <code>&lt;node&gt;[1].object</code> is the visibility controller or <code>undefined</code> if not yet assigned, <code>&lt;node&gt;[2].object</code> contains the bindings of any space warps bound to the node, or <code>undefined</code> if none have been bound, <code>&lt;node&gt;[3].object</code> is the transform controller, <code>&lt;node&gt;[4].object</code> is the node&apos;s object, either the modified object if there are modifiers present or the base object if not, <code>&lt;node&gt;[5].object</code> is the material assigned to the node or <code>undefined</code> if no material has been assigned, and <code>&lt;node&gt;[6].object</code> is the Image Motion Blur Multiplier controller or <code>undefined</code> if not yet assigned.</p>\n\
<blockquote>\n\
<p>   <strong>EXAMPLE</strong></p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">   b = box()\n\
   b[4].object -- returns a reference to the Box base object\n\
</code></pre></blockquote>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">&lt;subAnim&gt;.numSubs : Integer, read-only </code></pre><p>The number of immediate subAnim children.</p>\n\
<p>If you attempt to access subAnim properties, such as <code>.value</code> , <code>.numSubs</code> , <code>.category</code> , etc., on certain subAnims that had yet to be assigned controllers, such as scene node visibility tracks, these properties return null values such as 0, or <code>undefined</code> , as appropriate.</p>\n\
<p><strong>Methods</strong></p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">getSubAnim &lt;subAnim&gt; &lt;index&gt; </code></pre><p>Returns the indexed subAnim. See the index operator below.</p>\n\
<p><strong>Operators</strong></p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">&lt;subAnim&gt;[&lt;index_number&gt;] </code></pre><p>Returns the indexed subAnim. You can work down through nested subAnims (nested tracks) by indexing into subAnims themselves.</p>\n\
<blockquote>\n\
<p>   <strong>EXAMPLE</strong></p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">   $box01[3][2]\n\
</code></pre><p>   yields the rotation track subAnim since <node>[3] is always the transform track and the second track in it is always the rotation track. Indexing a SubAnim always yields another SubAnim. For those familiar with SubAnims in the <strong>3ds Max</strong> SDK, it should be noted that MAXScript SubAnims descend automatically through &apos;hidden&apos; subAnim levels, reflecting the structure seen in Track View.</node></p>\n\
</blockquote>\n\
<div class=\"note note:\"><span class=\"notetitle\">Note:</span> </div>\n\
<p>You can get at the base object&apos;s Track View parameters by indexing into the base object:</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">b=box()\n\
getSubAnimNames b[4] -- returns #(#length, #width, #height, ...)\n\
b[4][2].value -- returns current width value\n\
i = 3\n\
b[4][i].controller = float_list() -- assign height controller</code></pre>\n\
  </div>\n\
\n\
    <div class=\"related-links\">\n\
  \n\
    <div class=\"familylinks\">\n\
      <div class=\"parentLink\">\n\
        <strong>Parent page:</strong> <a class=\"link\" href=\"../Identifying-and-Accessing.html\">Identifying and Accessing MAXScript Classes and Properties</a>\n\
      </div>\n\
  \n\
    </div>\n\
  \n\
    </div>\n\
\n\
</body>\n\
</html>\n\
";