var topic = "<html>\n\
<head>\n\
	<script src=\"../scripts/lib/jquery-1.11.1.min.js\" type=\"text/javascript\"/></script\n\
	<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\">\n\
	<!--script type=\"text/javascript\" src=\"..\\../scripts/utils/adsk.redirect.js\"/></script>	\n\
	<script type=\"text/javascript\" src=\"..\\../scripts/prettify/lang-maxscript.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\../scripts/prettify/run_prettify.js\"/></script-->\n\
    <title>MAXScript Help</title>\n\
    <meta name=\"topicid\" content=\"GUID-C6A4DAEC-F65F-41DA-B82C-C87F3D1F7C59\" />\n\
    </head>\n\
<body height=\"100%\">\n\
  <a name=\"top\"></a>\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2022/ENU/MAXScript-Help/styles/bootstrap.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2022/ENU/MAXScript-Help/styles/additions.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2022/ENU/MAXScript-Help/styles/font-awesome.min.css\">\n\
      <script src=\"/cloudhelp/2022/ENU/MAXScript-Help/scripts/prettify/lang-maxscript.js\"></script>\n\
\n\
  <div class=\"body_content\" id=\"body-content\">\n\
  <div class=\"head-block\"><h1>The Wonderful World of Classes and Class Instances</h1></div><h2 id=\"a-user-asked-\">A user asked:</h2><p>I want to be able to turn RenderEffects on and off, but I have not been able to figure out if this is even possible.</p>\n\
<p>I am trying to decipher exactly how the Maxscript Help file syntax works. What does it mean when something is in the &lt; &gt; symbols as in:</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">setActive &lt;renderEffect&gt; &lt;boolean&gt;</code></pre><p>I understand that setActive is the command and boolean is either a true or false flag, but what is required with renderEffect? How does that command actually get written?</p>\n\
<p>I tried</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">setActive blur true\n\
setActive blur false\n\
</code></pre><p>This did not seem to be acceptable as the error states:</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">-- Runtime error: Invalid object, needed Atmospheric or Effect, got: blur</code></pre><p>So, what am I to provide in terms of an OOP object?</p>\n\
<h2 id=\"answer-\">Answer:</h2><p>According to <a href=\"./../../MAXScript-Introduction/Syntax-Definitions-in-This/Syntax-Definitions-in-This-1.html\">Syntax Definitions in This Document</a></p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">&lt;rule&gt; -- you can insert what is defined by the named rule</code></pre><p>So, when you see <code>&lt;boolean&gt;</code>, you need an expression which evaluates to a boolean value. This also means that it does not only expect the symbol True or False, but any expression leading to the value represented by the symbols True and False. So, in place of <code>&lt;boolean&gt;</code>, you can have a variable containing a boolean value, an expression like myValue &gt;= 10.5, or a MAXScript method that returns a boolean value (like selection[1].isHidden).</p>\n\
<p>In the case of <code>&lt;renderEffect&gt;</code>, it expects a renderEffect object value. As <strong>3ds Max</strong> is an Object-Oriented program, most of its building blocks are objects in the OOP meaning of the word. Similarly, MAXScript lets you construct instances of object classes.</p>\n\
<p>For example, b=box() creates an instance of the class Box and returns a value that is then stored in the variable b, which can be used anywhere where a <code>&lt;Node&gt;</code> is expected according to the MAXScript Help.</p>\n\
<p>If you create an instance of the lens_effects class, which is of renderEffect superClass, you get back an object that is not a scene node, but an object in the OOP meaning of the word and fits the rule mentioned above because its superClass matches the type of object expected by the setActive method.</p>\n\
<p>For example, try to see this from a human perspective:</p>\n\
<p>The CLASS of an object can be seen as the Nationality of a human.</p>\n\
<p>The SUPERCLASS can be seen as the species, in this case, HumanBeing.</p>\n\
<p>So, when you give birth to a human being in Bulgaria, for example, you are creating an instance of the class Bulgarian, of superclass HumanBeing.</p>\n\
<p>So, class and superclass are general descriptions of types of objects, but not the objects themselves. If you want to apply the method &quot;Kiss&quot; to a human being, you cannot use &quot;Kiss Bulgarian&quot; as it is not possible to kiss a nation. You need an <strong>instance</strong> of the class, a single human being, to call the method. So, in MAXScript pseudo-code:</p>\n\
<blockquote>\n\
<p>   <strong>PSEUDO-SCRIPT:</strong></p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">   theHuman = Bulgarian()--create an instance of the class Bulgarian\n\
   theHuman.name =&quot;Bobo&quot;--give it a name\n\
   print (superclassof theHuman)\n\
   --&gt; HumanBeing\n\
   if theHuman.location ==&quot;Winnipeg&quot; do\n\
   theHuman.isFrozen = true--set some properties based on object&apos;s location\n\
   -- Now you have a single instance you can call a method\n\
   -- that expects an object of the superclass HumanBeing:\n\
   Kiss theHuman\n\
</code></pre></blockquote>\n\
<p>If you try to use &quot;Kiss Bulgarian&quot; in <strong>Pseudo</strong> -MAXScript, you will get an error like:</p>\n\
<p>--Runtime error: Invalid object, needed HumanBeing, got: Bulgarian</p>\n\
<p>This is because Bulgarian is not an object. It is a class describing a category of objects.</p>\n\
<p>The idea of Object Oriented Programming is to have common classes of objects that inherit from more general classes, and lets you create any number of instances of a class that inherits all typical properties and capabilities of the class hierarchy above.</p>\n\
<p>So, you can have a class hierarchy like:</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">LifeForm &gt; EarthInhabitant &gt; Mammal &gt; HumanBeing &gt; Caucasian &gt; Bulgarian\n\
</code></pre><p>The more general classes in the beginning define common capabilities of a life form like it breathes air or swims in water, drinks milk, is a two-legged creature, the origin and some common skin, hair, and eye color properties, the language it speaks, the region it comes from, and so on.</p>\n\
<p>Still, you have to create an instance of the class you are interested in to use methods applicable to objects as shown above.</p>\n\
<p>In the special case of the RenderEffects, you need an instance of the Blur effect, not the class Blur itself. Usually, if the object exists in the scene, you can try to get the existing one into a variable, by calling:</p>\n\
<blockquote>\n\
<p>   <strong>FOR EXAMPLE</strong></p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">   myEffect = getEffect 1--returns the first effect on the RenderEffects list\n\
   setActive myEffect false--disables it\n\
</code></pre></blockquote>\n\
<p>Or shorter, without the intermediate variable.</p>\n\
<blockquote>\n\
<p>   <strong>FOR EXAMPLE</strong></p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">   setActive (getEffect 1) false\n\
</code></pre></blockquote>\n\
<p>To disable all effects in the scene, you can do something like:</p>\n\
<blockquote>\n\
<p>   <strong>FOR EXAMPLE</strong></p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">   for i = 1 to NumEffects do setActive (getEffect i) false\n\
</code></pre></blockquote>\n\
\n\
  </div>\n\
\n\
    <div class=\"related-links\">\n\
  \n\
    <div class=\"familylinks\">\n\
      <div class=\"parentLink\">\n\
        <strong>Parent page:</strong> <a class=\"link\" href=\"../MAXScript-Syntax-and-Terminology.html\">MAXScript Syntax and Terminology</a>\n\
      </div>\n\
  \n\
    </div>\n\
  \n\
      <div class=\"relinfo\">\n\
        <h4 class=\"related\">Related pages</h4>\n\
        <ul>\n\
          <li><a class=\"link\" href=\"../Frequently-Asked-Questions-1.html\">Frequently Asked Questions</a></li>\n\
        </ul>\n\
      </div>\n\
    </div>\n\
\n\
</body>\n\
</html>\n\
";