var topic = "<html>\n\
<head>\n\
	<script src=\"../scripts/lib/jquery-1.11.1.min.js\" type=\"text/javascript\"/></script\n\
	<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\">\n\
	<!--script type=\"text/javascript\" src=\"..\\..\\../scripts/utils/adsk.redirect.js\"/></script>	\n\
	<script type=\"text/javascript\" src=\"..\\..\\../scripts/prettify/lang-maxscript.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\..\\../scripts/prettify/run_prettify.js\"/></script-->\n\
    <title>MAXScript Help</title>\n\
    <meta name=\"topicid\" content=\"GUID-82AD9B41-AD84-4EF9-8CFB-01D2CF037A3F\" />\n\
    </head>\n\
<body height=\"100%\">\n\
  <a name=\"top\"></a>\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2022/ENU/MAXScript-Help/styles/bootstrap.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2022/ENU/MAXScript-Help/styles/additions.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2022/ENU/MAXScript-Help/styles/font-awesome.min.css\">\n\
      <script src=\"/cloudhelp/2022/ENU/MAXScript-Help/scripts/prettify/lang-maxscript.js\"></script>\n\
\n\
  <div class=\"body_content\" id=\"body-content\">\n\
  <div class=\"head-block\"><h1>How do I access UI elements to change their properties?</h1></div><h2 id=\"a-user-asked-\">A user asked:</h2><p>I have built a UI with buttons and progressBars. The progressBars are actually being used as small color indicators of availability. I want to change the color of the progress bars according to the files found.</p>\n\
<p>But I get &apos;undefined&apos; if I put it after the creation of the rollout with the progress bars. I want to change the color of these progress bars after they are created based on the data collected.</p>\n\
<h2 id=\"answer-\">Answer:</h2><p>Consider that you are creating a UI rollout with some progressbar in it.</p>\n\
<blockquote>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">   macroScript TestUI category:&quot;MXS Help&quot;\n\
   (\n\
     -- note that the mactoscript itself defines a scope\n\
     rollout testUI_Rollout&quot;Testing...&quot;\n\
     (\n\
       progressbar Indicator01 width:20 height:20 color:green value:100\n\
     )\n\
     createDialog testUI_Rollout 100 100\n\
   )\n\
</code></pre></blockquote>\n\
<p>You have three ways to access the Indicator01:</p>\n\
<ol>\n\
<li><p>Inside the rollout</p>\n\
</li>\n\
<li><p>Before the rollout is created</p>\n\
</li>\n\
<li><p>After the rollout is created</p>\n\
</li>\n\
</ol>\n\
<p>The first way is obvious and is not explained here.</p>\n\
<p>The second way is rather interesting because you can access values that are not yet defined, but are at least pre-initialized.</p>\n\
<blockquote>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">   macroScript TestUI category:&quot;MXS Help&quot;\n\
   (\n\
     local testUI_Rollout\n\
     -- this defines a variable\n\
     -- which has a memory address but still not valid value.\n\
     -- If you create a function that accesses the variable, the\n\
     -- evaluation will proceed as it sees the variable and assumes\n\
     -- it is a valid rollout. Later, when the actual execution of\n\
     -- the script happens, the variable will REALLY point at a valid\n\
     -- rollout, so everybody is happy!\n\
\n\
     fn changeTheLamp theColor =\n\
     (\n\
       testUI_Rollout.Indicator01.color = theColor\n\
       -- the function is outside of the rollout context,\n\
       -- so it has to prefix the UI element access.\n\
     )\n\
\n\
     -- This rollout is defined even before the rollout with the lamp\n\
     -- Still, it will be able to change the lamp to yellow using the\n\
     -- above function!\n\
     rollout Changer_Rollout&quot;Testing...&quot;\n\
     (\n\
       button changeIt&quot;CHANGE STATE&quot;\n\
       on changeIt pressed do changeTheLamp [255,255,0]\n\
     )\n\
\n\
     rollout testUI_Rollout&quot;Testing...&quot;\n\
     (\n\
       progressbar Indicator01 width:20 height:20 color:green value:100\n\
     )\n\
     createDialog Changer_Rollout 100 30 100 100\n\
     createDialog testUI_Rollout 100 100 100 200\n\
   )\n\
</code></pre></blockquote>\n\
<p>In the third way, consider that you want to access the value of Indicator01 or change its color after the dialog was created. In this case, because the variable TestUi_Rollout points at the rollout definition, you have to access Indicator01 by prefixing it with the rollout name:</p>\n\
<blockquote>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">   macroScript TestUI category:&quot;MXS Help&quot;\n\
   (\n\
     -- note that the mactoscript itself defines a scope\n\
     rollout testUI_Rollout &quot;Testing...&quot;\n\
     (\n\
       progressbar Indicator01 width:20 height:20 color:green value:100\n\
     )\n\
     createDialog testUI_Rollout 100 100\n\
     TestUI_rollout.Indicator01.color = red\n\
   )\n\
</code></pre></blockquote>\n\
<p>Note that the line is inside the scope of the macroScript where TestUI_rollout is visible. If you want to access the rollout from another script, you must define the rollout as a global variable:</p>\n\
<blockquote>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">   macroScript TestUI category:&quot;MXS Help&quot;\n\
   (\n\
     global testUI_Rollout --now the rollout is visible to anyone outside!\n\
     rollouttestUI_Rollout&quot;Testing...&quot;\n\
     (\n\
       progressbar Indicator01 width:20 height:20 color:green value:100\n\
     )\n\
     createDialog testUI_Rollout 100 100\n\
   )\n\
   macroScript ChangeStatusTestUI category:&quot;MXS Help&quot;\n\
   (\n\
     TestUI_rollout.Indicator01.color = red\n\
   )\n\
</code></pre></blockquote>\n\
<p>If you execute the first macro, a rollout with green light appears.</p>\n\
<p>If you execute the second one, the lamp turns red.</p>\n\
<p>As you can see, you can access anything from anywhere, provided that you have defined the respective rollouts as local or global variables in upper scopes to make them &quot;visible&quot;. Even better, you can declare the variables prior to defining their actual contents and access them before the point they are defined.</p>\n\
\n\
  </div>\n\
\n\
    <div class=\"related-links\">\n\
  \n\
    <div class=\"familylinks\">\n\
      <div class=\"parentLink\">\n\
        <strong>Parent page:</strong> <a class=\"link\" href=\"../Working-with-Scripted-UI.html\">Working with Scripted UI</a>\n\
      </div>\n\
  \n\
    </div>\n\
  \n\
      <div class=\"relinfo\">\n\
        <h4 class=\"related\">Related pages</h4>\n\
        <ul>\n\
          <li><a class=\"link\" href=\"../../Frequently-Asked-Questions-1.html\">Frequently Asked Questions</a></li>\n\
        </ul>\n\
      </div>\n\
    </div>\n\
\n\
</body>\n\
</html>\n\
";