var topic = "<html>\n\
<head>\n\
	<script src=\"../scripts/lib/jquery-1.11.1.min.js\" type=\"text/javascript\"/></script\n\
	<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\">\n\
	<!--script type=\"text/javascript\" src=\"..\\..\\../scripts/utils/adsk.redirect.js\"/></script>	\n\
	<script type=\"text/javascript\" src=\"..\\..\\../scripts/prettify/lang-maxscript.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\..\\../scripts/prettify/run_prettify.js\"/></script-->\n\
    <title>MAXScript Help</title>\n\
    <meta name=\"topicid\" content=\"GUID-A731E672-9B9A-4D5B-A8A7-EA584AD12FE5\" />\n\
    </head>\n\
<body height=\"100%\">\n\
  <a name=\"top\"></a>\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2022/ENU/MAXScript-Help/styles/bootstrap.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2022/ENU/MAXScript-Help/styles/additions.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2022/ENU/MAXScript-Help/styles/font-awesome.min.css\">\n\
      <script src=\"/cloudhelp/2022/ENU/MAXScript-Help/scripts/prettify/lang-maxscript.js\"></script>\n\
\n\
  <div class=\"body_content\" id=\"body-content\">\n\
  <div class=\"head-block\"><h1>Disable Undo system when possible</h1></div><p>The undo system can consume large amounts of memory and slow down processing.</p>\n\
<ul>\n\
<li><p>MAXScript commands run from Listener. Script Editor or Macro Script are run within an undo on context.</p>\n\
</li>\n\
<li><p>MAXScript commands run from scripted UI controls are <strong>not</strong> run within an undo on context.</p>\n\
</li>\n\
<li><p>Do not delete nodes with undo off, unless you also created the nodes with undo off and did not do anything to the node with undo on.</p>\n\
</li>\n\
<li><p>When working with meshes, you typically want to use meshop methods as they support undo/redo. But typically you do not want to store undo records for all operations in a loop, just the first and last.</p>\n\
</li>\n\
<li><p>When working with EPoly, you want to store undo records for all operations in a loop.</p>\n\
</li>\n\
</ul>\n\
<blockquote>\n\
<p>   <strong>TEST CASE:</strong></p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">   em = mesh()\n\
   meshop_setvert = meshop.setvert\n\
   fn test7 holdAll =\n\
   (\n\
    local nVerts = getnumverts em\n\
    for i = 1 to nVerts do\n\
     with undo (holdAll or (i == 1 or i == nVerts))\n\
      meshop_setvert em i ([1,1,1]*i)\n\
   )\n\
</code></pre></blockquote>\n\
<blockquote>\n\
<p>   <strong>FOR 100,000 ITERATIONS:</strong></p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">   test7 true -- 85313 msec., 229 MB\n\
   test7 false-- 7609 msec., 11 MB\n\
</code></pre></blockquote>\n\
<p>Every operation that creates an undo record will cost time and memory as it will create internal copies of the changing objects to allow an undo later. When making multiple changes in a loop like for example attaching multiple objects together, the Undo system would attempt to create a single undo copy of each resulting object and might even run out of memory. Disabling Undo explicitly using the <code>Undo off ()</code> context can help to speed up scripts significantly in such cases.</p>\n\
<p><a href=\"./../../../MAXScript-Language-Reference/Names-Literal-Constants-and/Context-Expressions/undo.html\">undo</a></p>\n\
<p>The following extreme example shows the difference. In both cases, 1000 boxes will be created using MAXScript and then attached to a single mesh using the attach function. In the first case, each attach call will generate by default an undo record. In the second case, the undo will be explicitly disabled.</p>\n\
<blockquote>\n\
<p>   <strong>EXAMPLE 1 - UNOPTIMIZED SCRIPT:</strong></p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">   delete $Box*--delete any existing boxes\n\
   box_array = #()--initialize an array\n\
   for i = 1 to 1000 do--repeat 1000 times\n\
     box_array[i] = box pos:[i*30,0,0]--create 1000 boxes\n\
   st = timestamp()--get the start time in milliseconds\n\
   main_box = convertToMesh box_array[1]--collapse the first box to mesh\n\
   for i = 2 to 1000 do--go through all other boxes\n\
     attach main_box box_array[i]--attach each box to the mesh\n\
   et = timestamp()--stop the time\n\
   print (et-st)--print the resulting time\n\
   gc()--call Garbage Collection &amp;endash; you will needed it!\n\
</code></pre><p>   On a 800MHz PC, the execution of the attaching part of the script took more than a minute mainly because the system run out of memory and the OS had to swap to disk. Memory usage for <strong>3ds Max</strong> went up with about 300 MB!</p>\n\
</blockquote>\n\
<blockquote>\n\
<p>   <strong>EXAMPLE 2 - OPTIMIZED SCRIPT:</strong></p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">   delete $Box*\n\
   box_array = #()\n\
   for i = 1 to 1000 do\n\
   box_array[i] = box pos:[i*30,0,0]\n\
   st = timestamp()\n\
   undo off--the only difference - the undo\n\
   (--has been turned off\n\
     main_box = convertToMesh box_array[1]\n\
   for i = 2 to 1000 do\n\
     attach main_box box_array[i]\n\
   )--end undo off\n\
   et = timestamp()\n\
   print (et-st)\n\
   --call Garbage Collection - you will needed it!\n\
</code></pre><p>   On the same 800MHz machine, the execution of the attaching part of the script took only 3685 milliseconds, almost 20 times faster! There were no memory consumption changes visible in the Windows Task Manager.</p>\n\
</blockquote>\n\
<p><strong>Previous Tip</strong></p>\n\
<p><a href=\"Disable-Viewport-Redraws-when.html\">Disable Viewport Redraws when making changes to scene objects</a></p>\n\
<p><strong>Next Tip</strong></p>\n\
<p><a href=\"Modify-Panel-can-be-slow-change.html\">Modify Panel can be slow -change to Create Panel when possible</a></p>\n\
\n\
  </div>\n\
\n\
    <div class=\"related-links\">\n\
  \n\
    <div class=\"familylinks\">\n\
      <div class=\"parentLink\">\n\
        <strong>Parent page:</strong> <a class=\"link\" href=\"../How-To-Make-It-Faster-.html\">How To Make It Faster?</a>\n\
      </div>\n\
  \n\
    </div>\n\
  \n\
    </div>\n\
\n\
</body>\n\
</html>\n\
";