var topic = "<html>\n\
<head>\n\
	<script src=\"../scripts/lib/jquery-1.11.1.min.js\" type=\"text/javascript\"/></script\n\
	<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\">\n\
	<!--script type=\"text/javascript\" src=\"..\\..\\../scripts/utils/adsk.redirect.js\"/></script>	\n\
	<script type=\"text/javascript\" src=\"..\\..\\../scripts/prettify/lang-maxscript.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\..\\../scripts/prettify/run_prettify.js\"/></script-->\n\
    <title>MAXScript Help</title>\n\
    <meta name=\"topicid\" content=\"GUID-8A616D27-6311-4A66-9C2D-43534D24511B\" />\n\
    </head>\n\
<body height=\"100%\">\n\
  <a name=\"top\"></a>\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2022/ENU/MAXScript-Help/styles/bootstrap.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2022/ENU/MAXScript-Help/styles/additions.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2022/ENU/MAXScript-Help/styles/font-awesome.min.css\">\n\
      <script src=\"/cloudhelp/2022/ENU/MAXScript-Help/scripts/prettify/lang-maxscript.js\"></script>\n\
\n\
  <div class=\"body_content\" id=\"body-content\">\n\
  <div class=\"head-block\"><h1>Using Bsearch For Fast Table Lookup</h1></div><p>The <a href=\"./../../../MAXScript-Language-Reference/Collections/Collection-Types/Array-Values.html\">bsearch()</a> method available in <strong>3ds Max 2009 and higher</strong> allows for very fast table lookup operations in sorted arrays.</p>\n\
<h2 id=\"the-problem\">The problem</h2><p>Assume that you have an array of arrays where each sub-array contains the same number of elements with the same layout. In many cases, you will collect this information just once and then retrieve the data stored in a specific sub-array (&quot;record&quot;) by providing a single search value (&quot;key&quot;) that represents the whole record without having to go through all objects in the scene again.</p>\n\
<h2 id=\"real-world-examples\">Real-World Examples</h2><p>An example is a list of all Editable Poly objects in the scene including their name, vertex, and poly count as well as the number of triangles, quad polygons, and n-gons in their geometry. As the collection of the polygon sides data is relatively slow (it requires looping through all polygons and determining their degree), you can collect this data just once and then be able to quickly ask &quot;How many tris and quads does the object with the name &quot;Box25&quot; contain?&quot;</p>\n\
<h2 id=\"using-maxscript-functions-for-linear-search\">Using MAXScript Functions for Linear Search</h2><p>Using a user-made MAXScript function to solve this problem will be relatively slow. It will require a linear search through the array (a loop) where the search key (in the above example, it is the object&apos;s name) is compared with the name stored in each sub-array of the table. If we assume that each entry is unique, then we can stop searching when we find a matching key. With this assumption the linear search will be very fast if the object in question happens to be the first record in the array, but much slower if it happens to be the last record. The average search time will be somewhere in between.</p>\n\
<blockquote>\n\
<p>   <strong>LINEAR SEARCH EXAMPLE:</strong></p>\n\
<p>   Below is one of the possible linear search functions used later on this page to compete against bsearch(). In short, the search key and the table to search is passed and it loops through them until it finds a match. At that point, it exits the loop and returns the current value.</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">   fn LookupMAXScriptLoop3 itemKey itemTable = (\n\
    for i in itemTable do if i[1] == itemKey do exit with i\n\
   ) \n\
</code></pre></blockquote>\n\
<h2 id=\"using-find-item-to-speed-up-maxscript-search\">Using Find Item To Speed Up MAXScript Search</h2><p>A possible acceleration of the MAXScript function search involves the creation of a second array containing only the keys from the original table and applying the findItem() method to it to get the index of the record. While this is technically also a form of linear search, findItem() is implemented in C++ and is much faster than a MAXScript For loop.</p>\n\
<blockquote>\n\
<p>   <strong>FINDITEM SEARCH EXAMPLE:</strong></p>\n\
<p>   Here, we pass the key, the table to search, and a table of all keys in the same order as the search table. FindItem() returns 0 if the key does not exist in the list or a positive index if it does. Using that index, we can return the record with the given key:</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">   fn LookupMAXScriptFindItem itemKey itemTable keyTable =(\n\
   local theIndex = findItem keyTable itemKey\n\
   if theIndex &gt; 0 then itemTable[theIndex] else undefined\n\
   )\n\
</code></pre></blockquote>\n\
<h2 id=\"using-bsearch-to-get-there-fast\">Using bsearch() To Get There FAST</h2><p>The bsearch() function solves the problem by working on a table that is already pre-sorted by the search key using qsort. It applies the same comparison function used for sorting to searching. As you see below, bsearch() scales great as the size of the table increases and is independent of the position of the record inside the table. It needs the same time to find the first, middle, or last record.</p>\n\
<p>The following test script can be used to test the bsearch() method against various variations of MAXScript functions.</p>\n\
<blockquote>\n\
<p>   <strong>TEST SCRIPT:</strong></p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">   (\n\
   -- Same compare function used by bsearch and qsort:\n\
   fn LookupTableComparator a b = (\n\
   if a[1] &gt; b[1] then 1\n\
   else if a[1] &lt; b[1] then -1\n\
   else 0\n\
   )\n\
\n\
   -- Finds an item in a lookup table; -- A lookup table is a sorted array of items, -- where each item is an array with its key as the same entry as the sort key. -- In our case the LookupTableComparator uses the first element as the key --So we have to make the search key an array too --in order to make it compatible with the qsort function!\n\
   fn LookupTableLookup itemKey itemTable = (\n\
   local lookupKey = #(itemKey)\n\
   bsearch lookupKey itemTable LookupTableComparator\n\
   )\n\
\n\
   --MAXScript Function1 -collects all matches and returns the first, if any\n\
   fn LookupMAXScriptLoop1 itemKey itemTable = (\n\
    result = for i in itemTable where i[1] == itemKey collect i\n\
   if result.count &gt; 0 then result[1] else undefined\n\
   )\n\
\n\
   --MAXScript Function2 -loops until a match is found and returns the result, if any\n\
   fn LookupMAXScriptLoop2 itemKey itemTable = (\n\
   for i in itemTable do if i[1] == itemKey do return i\n\
   undefined\n\
   )\n\
\n\
   --MAXScript Function3- loops until a match is found and exists with the result, if any\n\
   fn LookupMAXScriptLoop3 itemKey itemTable = (\n\
   for i in itemTable do if i[1] == itemKey do exit with i\n\
   )\n\
\n\
   --MAXScript Function loops using a while - managing the i var. is slower!\n\
   fn LookupMAXScriptLoop4 itemKey itemTable = (\n\
   local i = 0\n\
   result = undefined\n\
   while i &lt; itemTable.count do (\n\
   i+=1\n\
   if itemTable[i][1] == itemKey do (\n\
   result = itemTable[i]\n\
   i = itemTable.count + 1\n\
   )\n\
   )\n\
   result\n\
   )\n\
   fn LookupMAXScriptFindItem itemKey itemTable keyTable = (\n\
     local theIndex = findItem keyTable itemKey\n\
     if theIndex &gt; 0 then itemTable[theIndex] else undefined\n\
   )\n\
\n\
   --TEST VALUES:\n\
   local searchKey =&quot;5498&quot;--the key to search for\n\
   local iterations = 10--number of search calls\n\
   local tableElements = 10000--number of elements in the table\n\
\n\
   --Create an array to search in. --Its elements will have the form #(&quot;NNN&quot;, &quot;TestNNN&quot;, NNN). \n\
   --But the table could contain ANYTHING, this is just an example\n\
   searchTable =for i = 1 to tableElements collect\n\
   #( (i as string), &quot;Test&quot;+(i as string), i)\n\
\n\
   --For BSearch to work, the search table has to be sorted array!\n\
   st = timestamp()\n\
   qsort searchTable LookupTableComparator\n\
   format &quot;QSort Time: %ms\\n&quot; (timestamp()-st)\n\
\n\
   --Print the current first, middle and last element for best, mean and worst case searches\n\
   format &quot;First Element : %\\n&quot; searchTable[1]\n\
   format &quot;Mid. Element : %\\n&quot; searchTable[tableElements/2] \n\
   format &quot;Last Element : %\\n&quot; searchTable[tableElements]\n\
\n\
   local returnValue--variable to show the result\n\
\n\
   local st =timestamp()\n\
   for i = 1 to iterations do returnValue = LookupMAXScriptLoop1 searchKey searchTable\n\
   local totalTime =(timestamp()-st)\n\
   format &quot;MXS Loop 1 - Total: %ms Per Search: %ms\\n&quot; totalTime (1.0*totalTime/iterations)\n\
   format &quot;%\\n&quot; returnValue\n\
\n\
   st =timestamp()\n\
   for i = 1 to iterations do returnValue = LookupMAXScriptLoop2 searchKey searchTable\n\
   local totalTime =(timestamp()-st)\n\
   format &quot;MXS Loop 2 - Total: %ms Per Search: %ms\\n&quot; totalTime (1.0*totalTime/iterations)\n\
   format &quot;%\\n&quot; returnValue\n\
\n\
   st =timestamp()\n\
   for i = 1 to iterations do returnValue = LookupMAXScriptLoop3 searchKey searchTable\n\
   local totalTime =(timestamp()-st)\n\
   format &quot;MXS Loop 3 - Total: %ms Per Search: %ms\\n&quot; totalTime (1.0*totalTime/iterations)\n\
   format &quot;%\\n&quot; returnValue\n\
\n\
   st =timestamp()\n\
   for i = 1 to iterations do returnValue = LookupMAXScriptLoop4 searchKey searchTable\n\
   local totalTime =(timestamp()-st)\n\
   format &quot;MXS Loop 4 - Total: %ms Per Search: %ms\\n&quot; totalTime (1.0*totalTime/iterations)\n\
   format &quot;%\\n&quot;returnValue \n\
\n\
   local keyTable = (for i in searchTable collect i[1]) --collect all keys\n\
   local st =timestamp()\n\
   for i = 1 to iterations do\n\
   returnValue = LookupMAXScriptFindItem searchKey searchTable keyTable\n\
   local totalTime =(timestamp()-st)\n\
   format &quot;FindItem - Total: %ms Per Search: %ms\\n&quot; totalTime (1.0*totalTime/iterations)\n\
   format &quot;%\\n&quot;returnValue \n\
\n\
   st =timestamp()\n\
   for i = 1 to iterations do returnValue = LookupTableLookup searchKey searchTable\n\
   local totalTime =(timestamp()-st)\n\
   format &quot;BSEARCH - Total: %ms Per Search: %ms\\n&quot; totalTime (1.0*totalTime/iterations)\n\
   format &quot;%\\n&quot;returnValue \n\
   )\n\
</code></pre><p>   <strong>OUTPUT:</strong></p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">   QSort Time: 375ms\n\
   First Element : #(&quot;1&quot;, &quot;Test1&quot;, 1)\n\
   Mid. Element : #(&quot;5498&quot;, &quot;Test5498&quot;, 5498)\n\
   Last Element : #(&quot;9999&quot;, &quot;Test9999&quot;, 9999)\n\
   MXS Loop 1 - Total: 18812ms Per Search: 18.812ms\n\
   #(&quot;5498&quot;, &quot;Test5498&quot;, 5498)\n\
   MXS Loop 2 - Total: 9750ms Per Search: 9.75ms\n\
   #(&quot;5498&quot;, &quot;Test5498&quot;, 5498)\n\
   MXS Loop 3 - Total: 9594ms Per Search: 9.594ms\n\
   #(&quot;5498&quot;, &quot;Test5498&quot;, 5498)\n\
   MXS Loop 4 - Total: 18609ms Per Search: 18.609ms\n\
   #(&quot;5498&quot;, &quot;Test5498&quot;, 5498)\n\
   FindItem - Total: 1812ms Per Search: 1.812ms\n\
   #(&quot;5498&quot;, &quot;Test5498&quot;, 5498)\n\
   BSEARCH - Total: 0ms Per Search: 0.0ms\n\
   #(&quot;5498&quot;, &quot;Test5498&quot;, 5498)\n\
   OK\n\
</code></pre></blockquote>\n\
<p>In a table of 10,000 records sorted using qsort, key &quot;5498&quot; is exactly in the middle of the sorted table.</p>\n\
<p>The MXS Loop 1 performs as bad as in the worst case scenario of &quot;9999&quot; because it has to always loop through all elements.</p>\n\
<p>The MXS Loop 2 and 3 functions finish in half the time because they break in the middle of the loop. If we were searching for &quot;9999&quot;, their results will be close to those of MXS Loop 1. But, if we were searching for key &quot;1&quot;, they will be very fast because it is the first thing they hit when testing.</p>\n\
<p>The FindItem approach proves to be an order of magnitude faster.</p>\n\
<p>Finally, the BSEARCH method is so fast that we cannot even stop its time (less than 1 millisecond for 1000 calls).</p>\n\
<p>The following table shows the per search times with different table sizes and the best, mean, and worst cases for all methods. A large number of iterations (between 1000 and 100000 depending on the number of records) were used to ensure high precision.</p>\n\
<table>\n\
<thead>\n\
<tr>\n\
<th>Elements</th>\n\
<th>Key</th>\n\
<th>Loop 1</th>\n\
<th>Loop 2</th>\n\
<th>Loop 3</th>\n\
<th>Loop 4</th>\n\
<th>FindItem</th>\n\
<th>Bsearch</th>\n\
</tr>\n\
</thead>\n\
<tbody><tr>\n\
<td><strong>10</strong></td>\n\
<td>&quot;1&quot;</td>\n\
<td>0.01734ms</td>\n\
<td><strong>0.06266ms</strong></td>\n\
<td>0.04093ms</td>\n\
<td>0.00813ms</td>\n\
<td><strong>0.00375ms</strong></td>\n\
<td>0.01266ms</td>\n\
</tr>\n\
<tr>\n\
<td></td>\n\
<td>&quot;4&quot;</td>\n\
<td>0.01719ms</td>\n\
<td><strong>0.06781ms</strong></td>\n\
<td>0.04562ms</td>\n\
<td>0.01844ms</td>\n\
<td><strong>0.00421ms</strong></td>\n\
<td>0.00813ms</td>\n\
</tr>\n\
<tr>\n\
<td></td>\n\
<td>&quot;9&quot;</td>\n\
<td>0.01657ms</td>\n\
<td><strong>0.07438ms</strong></td>\n\
<td>0.05156ms</td>\n\
<td>0.03015ms</td>\n\
<td><strong>0.00469ms</strong></td>\n\
<td>0.01062ms</td>\n\
</tr>\n\
<tr>\n\
<td><strong>100</strong></td>\n\
<td>&quot;1&quot;</td>\n\
<td><strong>0.13031ms</strong></td>\n\
<td>0.06437ms</td>\n\
<td>0.04078ms</td>\n\
<td>0.00797ms</td>\n\
<td><strong>0.00406ms</strong></td>\n\
<td>0.02032ms</td>\n\
</tr>\n\
<tr>\n\
<td></td>\n\
<td>&quot;53&quot;</td>\n\
<td>0.12922ms</td>\n\
<td>0.12297ms</td>\n\
<td>0.10454ms</td>\n\
<td><strong>0.13016ms</strong></td>\n\
<td>0.00829ms</td>\n\
<td><strong>0.00812ms</strong></td>\n\
</tr>\n\
<tr>\n\
<td></td>\n\
<td>&quot;99&quot;</td>\n\
<td>0.12984ms</td>\n\
<td>0.18688ms</td>\n\
<td>0.16859ms</td>\n\
<td><strong>0.25812ms</strong></td>\n\
<td><strong>0.01329ms</strong></td>\n\
<td>0.01546ms</td>\n\
</tr>\n\
<tr>\n\
<td><strong>1000</strong></td>\n\
<td>&quot;1&quot;</td>\n\
<td><strong>1.3516ms</strong></td>\n\
<td>0.0609ms</td>\n\
<td>0.0391ms</td>\n\
<td>0.0078ms</td>\n\
<td><strong>0.0031ms</strong></td>\n\
<td>0.0328ms</td>\n\
</tr>\n\
<tr>\n\
<td></td>\n\
<td>&quot;548&quot;</td>\n\
<td><strong>1.4187ms</strong></td>\n\
<td>0.7156ms</td>\n\
<td>0.7000ms</td>\n\
<td>1.3000ms</td>\n\
<td>0.0547ms</td>\n\
<td><strong>0.0078ms</strong></td>\n\
</tr>\n\
<tr>\n\
<td></td>\n\
<td>&quot;999&quot;</td>\n\
<td>1.3875ms</td>\n\
<td>1.4250ms</td>\n\
<td>1.4156ms</td>\n\
<td><strong>2.7469ms</strong></td>\n\
<td>0.1063ms</td>\n\
<td><strong>0.0187ms</strong></td>\n\
</tr>\n\
<tr>\n\
<td><strong>10000</strong></td>\n\
<td>&quot;1&quot;</td>\n\
<td><strong>17.344ms</strong></td>\n\
<td>0.0620ms</td>\n\
<td>0.0470ms</td>\n\
<td>0.0160ms</td>\n\
<td><strong>0.0160ms</strong></td>\n\
<td>0.0470ms</td>\n\
</tr>\n\
<tr>\n\
<td></td>\n\
<td>&quot;5480&quot;</td>\n\
<td>17.250ms</td>\n\
<td>8.8590ms</td>\n\
<td>8.7970ms</td>\n\
<td><strong>20.187ms</strong></td>\n\
<td>1.6090ms</td>\n\
<td><strong>0.0310ms</strong></td>\n\
</tr>\n\
<tr>\n\
<td></td>\n\
<td>&quot;9999&quot;</td>\n\
<td>18.641ms</td>\n\
<td>18.531ms</td>\n\
<td>18.625ms</td>\n\
<td><strong>40.375ms</strong></td>\n\
<td>3.6870ms</td>\n\
<td><strong>0.0310ms</strong></td>\n\
</tr>\n\
<tr>\n\
<td><strong>100000</strong></td>\n\
<td>&quot;1&quot;</td>\n\
<td><strong>191.56ms</strong></td>\n\
<td>0.0630ms</td>\n\
<td>0.0470ms</td>\n\
<td>0.2650ms</td>\n\
<td><strong>0.0150ms</strong></td>\n\
<td>0.5940ms</td>\n\
</tr>\n\
<tr>\n\
<td></td>\n\
<td>&quot;54998&quot;</td>\n\
<td>194.38ms</td>\n\
<td>97.180ms</td>\n\
<td>96.570ms</td>\n\
<td><strong>3172.0ms</strong></td>\n\
<td>19.530ms</td>\n\
<td><strong>0.0150ms</strong></td>\n\
</tr>\n\
<tr>\n\
<td></td>\n\
<td>&quot;99999&quot;</td>\n\
<td>187.34ms</td>\n\
<td>187.03ms</td>\n\
<td>188.59ms</td>\n\
<td><strong>5659.22ms</strong></td>\n\
<td>38.750ms</td>\n\
<td><strong>0.2660ms</strong></td>\n\
</tr>\n\
</tbody></table>\n\
<p>As you can see from the table, the two best performers are the findItem and bsearch methods. With very few records in the array, findItem performs a bit faster.</p>\n\
<p>As the number of records in the array increases, the bsearch approach wins in both the average (searching for the middle record) and worst case scenarios (searching for the last record), while the linear search of findItem only wins in the best case scenario. Thus, if you have a very large list of records, for example 100,000, finding a random record by its key is 1300 times faster using bsearch() than by using findItem() and over 6400 times faster than the fastest of the MAXScript Linear Search (Loop) functions.</p>\n\
<p>The above times do not take into account the time necessary to sort the table using qsort() for bsearch() to be able to use, nor the time necessary to collect a keys array for the findItem() method. We assume that these are performed in a separate step once and then a very large number of random searches have to be performed very fast. The time needed to sort 100,000 records using qsort is around 30 seconds, which equals to about 800 calls to the findItem function. Thus, if you know that you will be calling the search function thousands of times, spending 30 seconds to pre-sort the list for bsearch() use is worth it and will pay off due to the much shorter search times later on.</p>\n\
<p><strong>Previous Tip</strong></p>\n\
<p><a href=\"Optimizing-Particle-Flow-Script.html\">Optimizing Particle Flow Script Operator For Loops For Speed</a></p>\n\
\n\
  </div>\n\
\n\
    <div class=\"related-links\">\n\
  \n\
    <div class=\"familylinks\">\n\
      <div class=\"parentLink\">\n\
        <strong>Parent page:</strong> <a class=\"link\" href=\"../How-To-Make-It-Faster-.html\">How To Make It Faster?</a>\n\
      </div>\n\
  \n\
    </div>\n\
  \n\
      <div class=\"relinfo\">\n\
        <h4 class=\"related\">Related pages</h4>\n\
        <ul>\n\
          <li><a class=\"link\" href=\"For-Loop-By-Index-Vs-For-Loop.html\">For Loop By Index Vs For Loop Through Collection Performance</a></li>\n\
        </ul>\n\
      </div>\n\
    </div>\n\
\n\
</body>\n\
</html>\n\
";