var topic = "<html>\n\
<head>\n\
	<script src=\"../scripts/lib/jquery-1.11.1.min.js\" type=\"text/javascript\"/></script\n\
	<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\">\n\
	<!--script type=\"text/javascript\" src=\"..\\../scripts/utils/adsk.redirect.js\"/></script>	\n\
	<script type=\"text/javascript\" src=\"..\\../scripts/prettify/lang-maxscript.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\../scripts/prettify/run_prettify.js\"/></script-->\n\
    <title>MAXScript Help</title>\n\
    <meta name=\"topicid\" content=\"GUID-8EC1EEA5-41CD-4E1F-B9FA-562B515EADE7\" />\n\
    </head>\n\
<body height=\"100%\">\n\
  <a name=\"top\"></a>\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2022/ENU/MAXScript-Help/styles/bootstrap.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2022/ENU/MAXScript-Help/styles/additions.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2022/ENU/MAXScript-Help/styles/font-awesome.min.css\">\n\
      <script src=\"/cloudhelp/2022/ENU/MAXScript-Help/scripts/prettify/lang-maxscript.js\"></script>\n\
\n\
  <div class=\"body_content\" id=\"body-content\">\n\
  <div class=\"head-block\"><h1>How To ... Generate Hash Checksums Using DotNet</h1></div><p>The following tutorials demonstrates the use of the <strong><code>.GetHashCode()</code></strong> method exposed by the <code>&quot;System.String</code> &quot; dotNetObject for determining whether specific data collected in the form of a string has changed between two runs of the script.</p>\n\
<p><strong>Related topics:</strong></p>\n\
<p><a href=\"./../../Interaction-with-Other/DotNet-In-MAXScript.html\">DotNet In MAXScript</a></p>\n\
<p><a href=\"./../../Interaction-with-Other/DotNet-In-MAXScript/dotnetobject.html\">dotNetObject</a></p>\n\
<blockquote>\n\
<p>   <strong>NATURAL LANGUAGE</strong></p>\n\
<p>   Define a MAXScript function to calculate the Hash code of all scene objects.</p>\n\
<p>   Collect the name, transformation matrix and parent of every object in a string.</p>\n\
<p>   Get the HashCode of the string and return it.</p>\n\
</blockquote>\n\
<blockquote>\n\
<p>   <strong>EXAMPLE</strong></p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">   fn getHashCodeFromGeometry = (\n\
     local hashStr = stringstream &quot;&quot;\n\
     for obj in objects do (\n\
       print obj.name to:hashStr\n\
       print obj.transform to:hashStr\n\
       if obj.parent != undefined then print obj.parent.name to:hashStr\n\
     )--end for loop\n\
     (dotNetObject &quot;System.String&quot; hashStr).GetHashCode()\n\
   )--end function\n\
</code></pre></blockquote>\n\
<h2 id=\"step-by-step\">Step-By-Step</h2><pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">fn getHashCodeFromGeometry = (</code></pre><p>Define a MAXScript function to be called by the Timer.</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">local hashStr = stringstream &quot;&quot;</code></pre><p>Create an empty string stream to collect the data to convert to Hash Code.</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">for obj in objects do (</code></pre><p>Loop through all scene objects...</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">print obj.name to:hashStr print obj.transform to:hashStr</code></pre><p>And print the name and transform of the current object to the stream.</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">if obj.parent != undefined then print obj.parent.name to:hashStr</code></pre><p>If the object has a parent, also print the parent&apos;s name to the stream.</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">)\n\
(dotNetObject &quot;System.String&quot; hashStr).GetHashCode()</code></pre><p>Finally, create an instance of the System.String class from the collected string stream and get its Hash Code.</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">)--end function</code></pre><h2 id=\"practical-use\">Practical Use</h2><p>Let&apos;s see how this function could be useful in a real-world situation.</p>\n\
<p>Let&apos;s assume we are developing a script which has to perform complex time consuming calculations when the scene content has changed. For example, it would have to rebuild all its data when</p>\n\
<ul>\n\
<li><p>the object count in the scene has changed (objects were added or removed),</p>\n\
</li>\n\
<li><p>one or more objects were renamed</p>\n\
</li>\n\
<li><p>one or more objects were moved, rotated or scaled</p>\n\
</li>\n\
<li><p>the hierarchy has changed (objects were linked or unlinked to/from parent objects)</p>\n\
</li>\n\
</ul>\n\
<p>It would be useful to cache the last results and return the same value next time the script is executed if the scene has not changed since the previous run. If checking the scene for changes is significantly faster than actually processing the data again, this would lead to a nice speed up.</p>\n\
<p>One possible approach would be to store all this scene state data in some user variables - assign the objects.count value to a variable, collect the names, transformation matrices and parent names of all objects into arrays the first time the script is run and then, the next time the script has to determine whether it needs to recalculate its initial data, it could collect the same information once again and compare all values with the results from the previous run.</p>\n\
<p>Obviously, if the scene contains 10,000+ objects, the amount of data to store (and the time needed to collect and compare the two data sets) would be quite substantial.</p>\n\
<p>But since we are not interested in the actual data values but only want to know whether they have changed between runs, we can use the HashCode value which represents ALL the above conditions with a single long integer value. All we have to do is run the function once to get a HashCode number reflecting the count, names, transforms and parents of all scene objects, then if we want to know if any of these properties have changed since the last time we run our script, we can call the function again and compare the two values. If the numbers are the same, the scene has not changed. In that case we can skip any data rebuilding steps since we can be confident the scene is in the same state as before. If the hash values are different, we must reprocess the scene to collect the actual data.</p>\n\
<p>Of course, the above function is just one example of possible HashCode usage. It can be changed to reflect more or less properties of objects, or could be used to track the state of other values that can be represented as strings.</p>\n\
\n\
  </div>\n\
\n\
    <div class=\"related-links\">\n\
  \n\
    <div class=\"familylinks\">\n\
      <div class=\"parentLink\">\n\
        <strong>Parent page:</strong> <a class=\"link\" href=\"../How-To-Develop-Tools-Using.html\">How To ... Develop Tools Using DotNet</a>\n\
      </div>\n\
  \n\
    </div>\n\
  \n\
      <div class=\"relinfo\">\n\
        <h4 class=\"related\">Related pages</h4>\n\
        <ul>\n\
          <li><a class=\"link\" href=\"../How-To-Introduction-to-the.html\">How To - Introduction to the Tutorials</a></li>\n\
        </ul>\n\
      </div>\n\
    </div>\n\
\n\
</body>\n\
</html>\n\
";