var topic = "<html>\n\
<head>\n\
	<script src=\"../scripts/lib/jquery-1.11.1.min.js\" type=\"text/javascript\"/></script\n\
	<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\">\n\
	<!--script type=\"text/javascript\" src=\"../scripts/utils/adsk.redirect.js\"/></script>	\n\
	<script type=\"text/javascript\" src=\"../scripts/prettify/lang-maxscript.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"../scripts/prettify/run_prettify.js\"/></script-->\n\
    <title>MAXScript Help</title>\n\
    <meta name=\"topicid\" content=\"GUID-E9BA617D-50E5-447F-997B-4F640F1110BA\" />\n\
    </head>\n\
<body height=\"100%\">\n\
  <a name=\"top\"></a>\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2022/ENU/MAXScript-Help/styles/bootstrap.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2022/ENU/MAXScript-Help/styles/additions.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2022/ENU/MAXScript-Help/styles/font-awesome.min.css\">\n\
      <script src=\"/cloudhelp/2022/ENU/MAXScript-Help/scripts/prettify/lang-maxscript.js\"></script>\n\
\n\
  <div class=\"body_content\" id=\"body-content\">\n\
  <div class=\"head-block\"><h1>Executing Python from MAXScript</h1></div><h2 id=\"using-python-libraries-in-maxscript\">Using Python Libraries in MAXScript</h2><p>Python libraries can be imported and used in MAXScript, using <code>Python.Import()</code>. In this example, we import the Python built-in methods and constants, and use the <code>pow()</code> function:</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">bi = Python.Import &quot;builtins&quot;\n\
--&gt; &lt;module &apos;builtins&apos; (built-in)&gt;\n\
bi.pow 2 3\n\
--&gt; 8</code></pre><p>If a module gets dirty (for example, a keyword gets re-defined), you can re-import using <code>Python.Reload()</code>:</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">Python.Reload bi\n\
--&gt; &lt;module &apos;builtins&apos; (built-in)&gt;</code></pre><p>Simple Python types, such as int, bool, float, and str, are copied into their corresponding MAXScript types. Complex types, such as lists, tuples, dictionaries and classes, are marshalled through a wrapper object. The properties and functions of the type are accessible by name. For sequence types such as a list, you can iterate using the index. Python lists passed to MAXScript are 1-based.</p>\n\
<p>For example, to create and access a Python tuple:</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">bi = Python.Import &quot;builtins&quot;\n\
tp = bi.tuple(#(1,2,3)) --(1, 2, 3)\n\
print tp[1] -- output: 1\n\
print tp[0] -- output: 3, same as tp[3]\n\
bi.type(tp) -- output: &lt;class &apos;tuple&apos;&gt;</code></pre><p>And to create and access a Python dict:</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">d = bi.dict one:1 two:2 three:3 -- {&apos;one&apos;: 1, &apos;two&apos;: 2, &apos;three&apos;: 3}\n\
d[&quot;four&quot;] = 4\n\
bi.print(d[&quot;four&quot;]) -- 4</code></pre><p>Here is an example of 1-based list passed to back to MAXScript from Python:</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">mystrlist = bi.str.split &quot;an example string&quot;\n\
-- [&apos;an&apos;, &apos;example&apos;, &apos;string&apos;]\n\
mystrlist[1]\n\
-- prints &quot;an&quot;</code></pre><h3 id=\"limitations\">Limitations</h3><p>Because of the way Python is wrapped in MAXScript, methods and functions bound to Python execution frames will not work as expected, if at all. Some examples of calls that will not work via <code>Python.import()</code> include:</p>\n\
<ul>\n\
<li><code>sys._current_frames</code></li>\n\
<li><code>sys._get_frame</code></li>\n\
<li><code>module.globals()</code></li>\n\
<li><code>module.locals()</code></li>\n\
</ul>\n\
<h2 id=\"executing-python-from-maxscript\">Executing Python from MAXScript</h2><p>You can execute Python commands and statements (can be a single command/expression or a series of statements); or a Python script (for example, a <code>.py</code> script) from within MAXScript. Use the <code>python.Execute</code> and <code>python.ExecuteFile</code> commands respectively, as follows:</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">python.Execute &quot;print(&apos;hello&apos;)&quot;</code></pre><pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">python.ExecuteFile &quot;demoBentCylinder.py&quot;</code></pre><p>If you provide a full path using the <code>ExecuteFile</code> command; for example, as follows; 3ds Max looks for the Python file in the path provided.</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">python.ExecuteFile @&quot;C:\\Program Files\\Autodesk\\3ds Max 2015\\scripts\\Python\\demoBentCylinder.py&quot;</code></pre><p>Otherwise, 3ds Max looks for the Python script under the following directories:</p>\n\
<ul>\n\
<li>User Scripts directory</li>\n\
<li>User Startup Scripts directory</li>\n\
<li>Scripts directory</li>\n\
<li>Startup Scripts</li>\n\
<li>Windows paths specified by the <code>Path</code> environment variable. To see a list of these directories, enter <code>echo %path%</code> in your command prompt.</li>\n\
</ul>\n\
<p>For example, 3ds Max would search under the following directories: <code>userscripts\\\\python</code>, <code>userscripts\\\\startup\\\\python</code>, <code>scripts\\\\python</code>, <code>scripts\\\\startup\\\\python</code>, and subsequently the windows paths.</p>\n\
<p>The output of the results from scripts are printed to the MAXScript Listener window.</p>\n\
<div class=\"note note:\"><span class=\"notetitle\">Note:</span>  Python scripts run in MAXScript are not thread-safe. Python commands are always executed in the main 3ds Max thread. You should not attempt to spawn separate threads in your scripts (for example, by using the Python threading module).</div>\n\
<h2 id=\"python-interface-methods\">Python interface methods</h2><p>The MAXScript Python interface contains methods for using Python in MAXScript. If you run</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">showinterface python</code></pre><p>in MAXScript, you obtain, in the MAXScript Listener, a list of available methods. The following list describes these properties, methods and arguments:</p>\n\
<ul>\n\
<li><code>Import</code>: Imports a Python module.</li>\n\
<li><code>Reload</code>: Reloads a Python module, for example if it has been re-defined.</li>\n\
</ul>\n\
\n\
<div class=\"callout callout-note\">\n\
<div class=\"callout-title\"><i class=\"callout-icon fa fa-pencil\" aria-hidden=\"true\"></i>Note:</div>\n\
<div class=\"callout-body\">\n\
\n\
<p> These are listed as properties in the Python interface, but they behave like methods in that they take an argument (the name of the Python module) and return a reference to the imported module.  See the examples above.</p>\n\
\n\
</div></div>\n\
\n\
<ul>\n\
<li><p><code>Init()</code>: Loads Python if it is not loaded already.</p>\n\
</li>\n\
<li><p><code>Execute()</code> and <code>ExecuteFile()</code>: Loads Python if it is not loaded already. Use <code>Init()</code> if you know that you will be using Python, and do not want the delay associated with loading it to occur the first time it is used. You would call <code>python.Init()</code> in a startup script to take the load time hit at startup, rather than while actually using 3ds Max.</p>\n\
</li>\n\
<li><p><code>GetLastError()</code>: Returns, as a string, the error message associated with the last error encountered while running the <code>Init()</code>, <code>Execute()</code>, or <code>ExecuteFile()</code> methods.</p>\n\
</li>\n\
<li><p>Result codes (<code>enums</code>): When executing Python from MAXScript, the result codes can be the following constants:</p>\n\
<ul>\n\
<li><code>#success</code></li>\n\
<li><code>#pathError</code></li>\n\
<li><code>#initError</code></li>\n\
<li><code>#scriptFileError</code></li>\n\
<li><code>#executeError</code></li>\n\
</ul>\n\
</li>\n\
<li><p><code>throwOnError</code> keyword argument: Controls whether MAXScript throws a runtime exception if an error occurs while running a method. If set to <code>true</code>, you always obtain a return result code of <code>#success</code> because, in all other cases, a runtime exception would be thrown. It would be up to the script writer to make the call within a try/catch in order to handle the exception.</p>\n\
</li>\n\
</ul>\n\
\n\
  </div>\n\
\n\
    <div class=\"related-links\">\n\
  \n\
    <div class=\"familylinks\">\n\
      <div class=\"parentLink\">\n\
        <strong>Parent page:</strong> <a class=\"link\" href=\"../Interaction-with-Other.html\">Interaction with Other Programming Languages and Environments</a>\n\
      </div>\n\
  \n\
    </div>\n\
  \n\
    </div>\n\
\n\
</body>\n\
</html>\n\
";