var topic = "<html>\n\
<head>\n\
	<script src=\"../scripts/lib/jquery-1.11.1.min.js\" type=\"text/javascript\"/></script\n\
	<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\">\n\
	<!--script type=\"text/javascript\" src=\"..\\../scripts/utils/adsk.redirect.js\"/></script>	\n\
	<script type=\"text/javascript\" src=\"..\\../scripts/prettify/lang-maxscript.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\../scripts/prettify/run_prettify.js\"/></script-->\n\
    <title>MAXScript Help</title>\n\
    <meta name=\"topicid\" content=\"GUID-A6E95530-F419-4837-B10A-E881A62AD5F9\" />\n\
    </head>\n\
<body height=\"100%\">\n\
  <a name=\"top\"></a>\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2022/ENU/MAXScript-Help/styles/bootstrap.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2022/ENU/MAXScript-Help/styles/additions.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2022/ENU/MAXScript-Help/styles/font-awesome.min.css\">\n\
      <script src=\"/cloudhelp/2022/ENU/MAXScript-Help/scripts/prettify/lang-maxscript.js\"></script>\n\
\n\
  <div class=\"body_content\" id=\"body-content\">\n\
  <div class=\"head-block\"><h1>Local and Global Variables</h1></div><p>In MAXScript, there are two classes of variables that you can create: &quot;local&quot; and &quot;global&quot; variables. When a variable is global, it means that the variable can be used anywhere in the program. The only place you cannot use a global variable is before you have defined it.</p>\n\
<blockquote>\n\
<p>   <strong>For example, the following lines will produce an error:</strong></p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">   sphere radius:rad\n\
   global rad = 10</code></pre></blockquote>\n\
<p>The variable &quot;<code>rad</code>&quot; does not exist until it has been defined.</p>\n\
<blockquote>\n\
<p>   <strong>The following is an acceptable form of the previous lines:</strong></p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">   global rad = 10\n\
   sphere radius:rad</code></pre></blockquote>\n\
<p>Now that &quot;<code>rad</code>&quot; has been defined, MAXScript was able to use it in the definition of a sphere. In fact, since &quot;<code>rad</code>&quot; is a global variable, it can be used throughout the rest of the script.</p>\n\
<p>There are several global variables that <em>can</em> be used without defining them first.</p>\n\
<p>They are the system global variables, which have already been created and assigned to a default value. Most of these variables can be reassigned to different values. For more information on the system global variables, see the <a href=\"./../../MAXScript-Language-Reference/Reserved-Keywords-Symbols/3ds-Max-System-Globals.html\">3ds Max System Globals</a> topic in the MAXScript reference.</p>\n\
<p>Unlike global variables, local variables can be used only within the block in which they are defined.</p>\n\
<p>A block is a grouping of MAXScript code, such as the statements within loops and conditional statements.</p>\n\
<blockquote>\n\
<p>   <strong>HERE IS AN EXAMPLE OF A FOR LOOP</strong></p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">   for i = 1 to 3 do\n\
   (\n\
     local rad = 10\n\
     s = sphere()\n\
     s.pos.x = i * 10\n\
     s.radius = rad\n\
   )</code></pre></blockquote>\n\
<p>Local variables can only exist within a block. For this reason, they must be created in a block. If you try to define a local variable from the top level of MAXScript, you will receive an error message.</p>\n\
<p>In the previous example, you defined a local variable named &quot;<code>rad</code>&quot;. This variable is active only within the parentheses inside the for loop; if you attempt to use the variable &quot;<code>rad</code>&quot; elsewhere, you will get an error.</p>\n\
<p>Although <code>rad</code> is initialized to a specific value, this is not necessary. When you define variables using a &quot;local&quot; or &quot;global&quot; identifier, you can do so without assigning an initial value. The name of the variable will be recognized. However, the software will not know the type of the variable (name, number, color, etc.). MAXScript automatically assigns the variable a special value of &quot;undefined&quot;. The first time you assign something to the variable, it will become defined by the data type that you have assigned it to. In practice, it is always best to define variables with an initial value.</p>\n\
<p>It is important to note that <code>rad</code> is not the only local variable in the previous example. The variable &quot;<code>s</code>&quot; is also a local variable, and cannot be accessed outside of the parentheses. It is not necessary to declare every variable with a &quot;global&quot; or &quot;local&quot; statement. You learned previously that you can implicitly create a variable by simply assigning a value to it. If you do not make the &quot;global&quot; or &quot;local&quot; distinction upon your variable, MAXScript will create the appropriate association, depending on where the variable is defined in the script. For example if you create a variable inside a block, MAXScript automatically treats it as a local variable. On the other hand, any variable created at the top level of MAXScript (outside of all blocks) is always a global variable. As mentioned previously, MAXScript does not allow for the creation of local variables outside of a block.</p>\n\
<p>These concepts are presented in the following</p>\n\
<blockquote>\n\
<p>   <strong>HERE IS AN EXAMPLE OF A FOR LOOP</strong></p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">   for i = 1 to 5 do\n\
   (\n\
     global rad = 10\n\
     cyl_height = 25\n\
     c = cylinder()\n\
     c.radius = rad\n\
     c.height = cyl_height\n\
     c.pos.x = i * 25\n\
   )\n\
   s = sphere radius:rad</code></pre></blockquote>\n\
<p>In this example, four variables are created: <code>rad, cyl_height, c</code>, and <code>s</code>. Because they were implicitly created within a block, <code>c</code> and <code>cyl_height</code> are both local variables. The other two variables are both global; <code>s</code>, because it was created outside of a block, and <code>rad</code>, because it was declared as a global variable. For this reason, you can use<code>rad</code> again to define the sphere, <code>s</code>.</p>\n\
<p><strong>Next Topic</strong></p>\n\
<p><a href=\"Defining-Custom-Functions.html\">Defining Custom Functions</a></p>\n\
\n\
  </div>\n\
\n\
    <div class=\"related-links\">\n\
  \n\
    <div class=\"familylinks\">\n\
      <div class=\"parentLink\">\n\
        <strong>Parent page:</strong> <a class=\"link\" href=\"../Creating-Your-Own-Scripts.html\">Creating Your Own Scripts</a>\n\
      </div>\n\
  \n\
    </div>\n\
  \n\
    </div>\n\
\n\
</body>\n\
</html>\n\
";