var topic = "<html>\n\
<head>\n\
	<script src=\"../scripts/lib/jquery-1.11.1.min.js\" type=\"text/javascript\"/></script\n\
	<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\">\n\
	<!--script type=\"text/javascript\" src=\"..\\../scripts/utils/adsk.redirect.js\"/></script>	\n\
	<script type=\"text/javascript\" src=\"..\\../scripts/prettify/lang-maxscript.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\../scripts/prettify/run_prettify.js\"/></script-->\n\
    <title>MAXScript Help</title>\n\
    <meta name=\"topicid\" content=\"GUID-821E791F-6D80-48C7-8A65-65BE73C551C6\" />\n\
    </head>\n\
<body height=\"100%\">\n\
  <a name=\"top\"></a>\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2022/ENU/MAXScript-Help/styles/bootstrap.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2022/ENU/MAXScript-Help/styles/additions.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2022/ENU/MAXScript-Help/styles/font-awesome.min.css\">\n\
      <script src=\"/cloudhelp/2022/ENU/MAXScript-Help/scripts/prettify/lang-maxscript.js\"></script>\n\
\n\
  <div class=\"body_content\" id=\"body-content\">\n\
  <div class=\"head-block\"><h1>Assert Functions</h1></div><p>You can use the Assert function to place assertions in the MAXScript code. Additionally, an <a href=\"Assert-Functions.html\"\">advanced set of assert functions and an AssertReporter message log system</a> are available, and are being used for test suites by Autodesk during the development of <strong>3ds Max</strong>.</p>\n\
<p><strong>Overview:</strong></p>\n\
<p>An assertion is a predicate (a true|false statement) placed in the code to indicate that the developer thinks the predicate always evaluates to true at that place.</p>\n\
<p>Assertions are used to help reason about program correctness.</p>\n\
<p><strong>Syntax:</strong></p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">assert &lt;expression&gt; [message:&lt;string&gt;] [options:#( #dialog | #listener | #debug_output_window | #debugBreak | #c_break | #all)] [showstack:&lt;boolean&gt;]</code></pre><p><strong>Asserts</strong> that the <strong>result of the expression</strong> passed as the first argument to the function <strong>must be true</strong> at the given place in the code.</p>\n\
<p>The expression <strong>must</strong> resolve to a Boolean value, otherwise a MAXScript exception is thrown.</p>\n\
<p>If the expression returns <code>true</code>, the code continues normally.</p>\n\
<p>If the expression returns <code>false</code>, the execution will be interrupted depending on the following options:</p>\n\
<p><code>message:</code> - If this optional keyword is supplied, the string is displayed as the assert&apos;s message.</p>\n\
<p><code>options:</code> - An array of the following MAXScript name options. If the array is empty, it defaults to <code>#listener</code>.</p>\n\
<p><code>#dialog</code>- brings up a modal dialog that halts execution of the application.</p>\n\
<p><code>#listener</code>  - prints out the failure info to theMAXScript Listener</p>\n\
<p><code>#debug_output_window</code> - prints out to the Visual Studio output debugger window, (if present)</p>\n\
<p><code>#debugBreak</code>   - Halts execution in the MAXScript Debugger. If specified, the #dialog option is ignored. This option is only valid if quiet mode is false. Otherwise nothing happens.</p>\n\
<p><code>#c_break</code>  - Breaks execution in Visual Studio under the Debugger. This only works if running a debug build of <strong>3ds Max</strong> under the Debugger. If specified, it first turns on the <code>#debug_output_window</code> flag internally to aid in debugging. The <code>#debugBreak</code> and <code>#dialog</code> option will be ignored and it always occurs last to allow the other options to output their data.</p>\n\
<p><code>#all</code>  - All of the above, except <code>#dialog</code> and <code>#c_break</code></p>\n\
<p><code>showstack:</code> - defines whether to include the MAXScript stack traceback in the printouts. This only applies for the <code>#listener</code> and <code>#debug_output_window</code> options. For all other options the stack information is ignored. Default is false.</p>\n\
<blockquote>\n\
<p>   <strong>EXAMPLE:</strong></p>\n\
<p>   In the following example, the function loops through all objects in the scene and accumulates their positions. At the end, it divides the resulting value by the number of objects to calculate the average position.</p>\n\
<p>   The script developer has assumed that the scene will never be empty, thus the final result would be a valid position. If the number of objects is zero, the division by zero would not crash the program, but would produce a Point3 value of[-1.#IND,-1.#IND,-1.#IND].</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">   fn averageObjectPosition =\n\
   (\n\
    theSum = [0,0,0]\n\
    theCount = objects.count\n\
    for i = 1 to theCount do\n\
    (\n\
     theSum += objects[i].pos\n\
    )\n\
    assert (theCount &gt; 0) message: &quot;theCount expected to be positive, but was found not to be!&quot; options:#(#all) showstack:true\n\
    theSum /= theCount\n\
   )\n\
\n\
   averageObjectPosition()\n\
</code></pre><p>   The developer has opted to include an <code>assert()</code> call, which postulates that <code>theCount</code> is expected to be always greater than zero and if that is not the case, the execution of the code will break in the debugger and will show the stack in the Listener.</p>\n\
<p>   In this example, this is done for demonstration and to illustrate how the <code>assert()</code> function works.</p>\n\
<p>   In real world, the <code>assert()</code> would be typically used to ensure the correctness of values going through much more complex calculations where catching irregularities with IF tests would be difficult because the developer cannot anticipate what can be the possible cases to test. In addition, the <code>assert()</code> has several report modes that could be useful (like breaking in the C++ Debugger when running a Debug build of <strong>3ds Max</strong>).</p>\n\
<p>   If the above function is executed with an empty scene, the result would be</p>\n\
<p>   <img src=\"./images/GUID-A940F022-D1F8-4820-920D-58B521648984-low.gif\" alt></p>\n\
</blockquote>\n\
<h2 id=\"test-framework-assert-functions\">Test Framework Assert Functions</h2><p>A number of assert functions and an <code>AssertReporter</code> struct to manage their messages have been implemented and are now being used for internal testing at Autodesk.</p>\n\
<p>These functions are available to all MAXScript users and can be employed in custom test frameworks to ensure the correctness of the program flow.</p>\n\
<p>These Test Framework functions have the following advantages:</p>\n\
<ol>\n\
<li>They report the file</li>\n\
<li>They report the line number</li>\n\
<li>They report the calling function</li>\n\
<li>They report the expected and actual values</li>\n\
</ol>\n\
<p>All asserts are reported automatically.</p>\n\
<p>This new system should significantly reduce the visual clutter of the asserts as developers have the option of NOT having to construct massive error strings for the message of the asserts (although they still can provide custom messages where needed). Also, these asserts run quietly and gather their messages in a dedicated <code>AssertReporter</code> to be queried and analyzed after running the code without interrupting the actual test run.</p>\n\
<p><strong>The assert functions are:</strong></p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">&lt;boolean&gt;assert_true &lt;expression&gt;expectedToBeTrue [message:&lt;string&gt;]</code></pre><p>Returns false and logs an assert failure message with the <code>AssertReporter</code> if the return value of the expression is not true.</p>\n\
<p>Will log a default message in the form of <strong>*&quot;Assert - Expected true, got false - (No File) in frame <no stack frame>, at line 1&quot;*</no></strong> unless the optional <code>message:</code> keyword argument is provided.</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">&lt;boolean&gt;assert_false &lt;expression&gt;expectedToBeFalse [message:&lt;string&gt;]</code></pre><p>Returns false and logs an assert failure message with the <code>AssertReporter</code> if the return value of the expression is not false.</p>\n\
<p>Will log a default message in the form of <strong>*&quot;Assert - Expected false, got true - (No File) in frame <no stack frame>, at line 1&quot;*</no></strong> unless the optional <code>message:</code> keyword argument is provided.</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">&lt;boolean&gt;assert_equal &lt;value&gt;expected &lt;expression&gt;actual [message:&lt;string&gt;]</code></pre><p>Returns false and logs an assert failure message with the <code>AssertReporter</code> if the return value of the expression passed as second argument is not equal to the expected value passed as first argument.</p>\n\
<p>Will log a default message in the form of <strong>*&quot;Assert - Expected 1, got 2 - (No File) in frame <no stack frame>, at line 1&quot;*</no></strong> unless the optional <code>message:</code> keyword argument is provided.</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">&lt;boolean&gt;assert_not_equal &lt;value&gt;expected &lt;expression&gt;actual [message:&lt;string&gt;]</code></pre><p>Returns false and logs an assert failure message with the <code>AssertReporter</code> if the return value of the expression passed as second argument is equal to the expected value passed as first argument.</p>\n\
<p>Will log a default message in the form of <strong>*&quot;Assert - Expected value not equal to <expected>, got <actual> - (No File) in frame <no stack frame>, at line 1&quot;*</no></actual></expected></strong> unless the optional <code>message:</code> keyword argument is provided.</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">&lt;boolean&gt;assert_defined &lt;expression&gt;expectedToBeDefined [message:&lt;string&gt;]</code></pre><p>Returns false and logs an assert failure message with the <code>AssertReporter</code> if the returns value of the expression is undefined.</p>\n\
<p>Will log a default message in the form of <strong>*&quot;Assert - Expected &quot;defined&quot;, got undefined - (No File) in frame <no stack frame>, at line 1&quot;*</no></strong> unless the optional <code>message:</code> keyword argument is provided.</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">&lt;boolean&gt;assert_undefined &lt;expression&gt;expectedToBeUndefined [message:&lt;string&gt;]</code></pre><p>Returns false and logs an assert failure message with the <code>AssertReporter</code> if the returns value of the expression is not undefined.</p>\n\
<p>Will log a default message in the form of <strong>*&quot;Assert - Expected undefined, got 1 - (No File) in frame <no stack frame>, at line 1&quot;*</no></strong> unless the optional <code>message:</code> keyword argument is provided.</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">&lt;boolean&gt;assert_float &lt;float&gt;expected &lt;expression&gt;actual [tolerance:&lt;float&gt;delta] [message:&lt;string&gt;]\n\
&lt;boolean&gt;assert_float_equal &lt;float&gt;expected &lt;expression&gt;actual [tolerance:&lt;float&gt;delta] [message:&lt;string&gt;]                                                                                                          </code></pre><p>Returns false and logs an assert failure message with the <code>AssertReporter</code> if the returns value of the expression passed as second argument is not the expected float value.</p>\n\
<p>If the <strong><code>tolerance:</code></strong> optional keyword argument is provided, the assert will return false if the actual return value is different from the expected one beyond the tolerance delta value.</p>\n\
<p>Will log a default message in the form of <strong>*&quot;Assert - Expected 1.0, got 2 - (No File) in frame <no stack frame>, at line 1&quot;*</no></strong> unless the optional <code>message:</code> keyword argument is provided.\n\
The <code>assert_float_equal()</code> version is identical to <code>assert_float()</code>, but is provided as it has a more meaningful name that matches the other assert functions.<br> <strong>Available in 3ds Max 2021.1 Update and higher.</strong></p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">&lt;boolean&gt;assert_string_equal &lt;string&gt;expected &lt;exression&gt;actual [ignorecase:&lt;boolean&gt;val]\n\
[message:]</code></pre><p>Returns false if the return value of the expression passed as second argument is not equal to the expected string.</p>\n\
<p>The default for the optional <code>ignorecase:</code> keyword argument is true. It should be supplied as false to perform case sensitive comparison.</p>\n\
<p>Will log a default message in the form of <strong>*&quot;Assert - Expected &quot;A&quot;, got &quot;b&quot; - (No File) in frame <no stack frame>, at line 1&quot;*</no></strong> unless the optional <code>message:</code> keyword argument is provided.</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">&lt;boolean&gt;assert_point3_equal &lt;point3&gt;expected &lt;expression&gt;actual [tolerance:&lt;float&gt;delta]\n\
[message:]</code></pre><p>Returns false if the return value of the expression passed as second argument is not equal to the expected Point3 value.</p>\n\
<p>If the <strong><code>tolerance:</code></strong> optional keyword argument is provided, the assert will return false if any of the actual return value&apos;s components is different from the expected ones&apos; corresponding component beyond the tolerance delta value.</p>\n\
<p>Will log a default message in the form of <strong>*&quot;Assert - Expected [1,2,3], got [1.1,2.1,3.1] - (No File) in frame <no stack frame>, at line 1&quot;*</no></strong> unless the optional <code>message:</code> keyword argument is provided.</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">&lt;boolean&gt;assert_matrix_equal &lt;matrix&gt;expected &lt;matrix&gt;actual [tolerance:&lt;float&gt;delta]\n\
[message:]</code></pre><p>Returns false if the return value of the expression passed as second argument is not equal to the expected matrix value.</p>\n\
<p>If the <strong><code>tolerance:</code></strong> optional keyword argument is provided, the assert will return false if any of the components of the actual return value is different from the expected ones&apos; corresponding component beyond the tolerance delta value.</p>\n\
<p>Will log a default message in the form of <strong>*&quot;Assert - Expected (matrix3 [1,0,0] [0,1,0] [0,0,1] [0,0,0]), got (matrix3 [1,0,0] [0,1,0] [0,0,1] [42.1234,43.8329,0]) - (No File) in frame <no stack frame>, at line 1&quot;*</no></strong> unless the optional <code>message:</code> keyword argument is provided.</p>\n\
<p>Note that due to the limited precision of numbers representation in computers, values at the exact edge of the tolerance delta might or might not cause an assert to be logged when using the <code>assert_float()</code> and <code>assert_point3_equal()</code> methods.</p>\n\
<blockquote>\n\
<p>   <strong>EXAMPLE:</strong></p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">   floatVariable = 1.0\n\
   assert_float 1.1 floatVariable tolerance:0.1\n\
   --&gt;&quot;Assert - Expected 1.1, got 1.0 ...&quot;</code></pre><p>   But when using the following code, there is no assert:</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">   point3Variable = [1,2,3]\n\
   assert_point3_equal [1.0,2.0,3.1] point3Variable tolerance:0.1</code></pre></blockquote>\n\
<p>The explanation for the above results is that the first method checks whether the difference of the expected and actual values is less than the tolerance value and in this case, it is not. The two values are converted to doubles internally and the first 7 decimal places are identical (all zeros), so the difference is NOT less than the tolerance delta.</p>\n\
<p>In the second case though, the Point3 value [1.0,2.0,3.1] is actually represented internally as [1.0,2.0,3.0999999]. Thus the difference between 3.0 and 3.0999999 is just below the tolerance and due to this precision error, the assert passes as true.</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">&lt;boolean&gt;assert_array_equal expected actual [message:&lt;string&gt;] [tolerance:&lt;boolean&gt;] [ignoreCase:&lt;boolean&gt;]</code></pre><p>Returns false if the return value of the expression passed as second argument is not equal to the expected array.\n\
 <strong>Available in 3ds Max 2021.1 Update and higher.</strong>\n\
The default for the optional <code>ignorecase:</code> keyword argument is true. It should be supplied as false to perform case sensitive comparison for arrays that contain strings.</p>\n\
<p>If the <strong><code>tolerance:</code></strong> optional keyword argument is provided, the assert will return false if any of the numeric elements of the actual return value is different from the expected ones&apos; corresponding elements beyond the tolerance delta value.</p>\n\
<blockquote>\n\
<p>   <strong>EXAMPLE:</strong></p>\n\
<p>   The following will all return false:</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">   assert_array_equal #(1) #(1.)\n\
   assert_array_equal #(1) #(1,2)\n\
   assert_array_equal #(1,2) #(1)\n\
   assert_array_equal #(1) #(2)\n\
   assert_array_equal #(1.) #(2.)\n\
   assert_array_equal #([1,2]) #([1,3])\n\
   assert_array_equal #([1,2,3]) #([1,2,4])\n\
   assert_array_equal #([1,2,3,4]) #([1,2,3,5])\n\
   assert_array_equal #(matrix3 1) #(transmatrix [1,1,1])\n\
   assert_array_equal #(&quot;aa&quot;) #(&quot;ab&quot;)\n\
   assert_array_equal #(1.) #(2.) tolerance:0.5\n\
   assert_array_equal #(&quot;aa&quot;) #(&quot;AA&quot;) ignoreCase:false</code></pre></blockquote>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">&lt;boolean&gt;assert_matchpattern expected actual [message:&lt;string&gt;] [ignoreCase:&lt;boolean&gt;]</code></pre><p><img src=\"./images/GUID-21C87158-BBA7-4B63-B8F7-CB9BA17CA2CA-low.gif\" alt> <a href=\"./../../What-is-New-in-MAXScript/What-s-New-in-MAXScript-in-3ds.html\">NEW</a> <strong>in 3ds Max 2021.1 Update:</strong> Returns false if the return value of the expression passed as second argument is not equal to the expected string. The second argument (<code>actual</code>) can be a match pattern, where ? represents a single character and * represents one or more characters.</p>\n\
<p>The default for the optional <code>ignorecase:</code> keyword argument is true. It should be supplied as false to perform case sensitive comparison for the strings.</p>\n\
<blockquote>\n\
<p>   <strong>EXAMPLE:</strong></p>\n\
<p>   The following will all return false:</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">   s=&quot;text1&quot;\n\
   assert_matchpattern &quot;tex?&quot; s\n\
   assert_matchpattern &quot;T*&quot; s ignoreCase:false</code></pre></blockquote>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">&lt;boolean&gt;assert_bitarray_equal expected_bitarray actual_bitarray [message:&lt;string&gt;]</code></pre><p>Returns false if the return value of the expression passed as second argument is not equal to the expected array.<br> <strong>Available in 3ds Max 2021.1 Update and higher.</strong>### The <code>AssertReporter</code> Struct</p>\n\
<p>The global <code>AssertReporter</code> C++ struct collects the results of the asserts and provides methods for managing and accessing the results after a test run.</p>\n\
<p><strong>General Methods:</strong></p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">&lt;void&gt;AssertReporter.Clear()</code></pre><p>Removes all assertion entries from the <code>AssertReporter</code> and resets all failures and exceptions results to zero.</p>\n\
<p><strong>Reporting Methods:</strong></p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">&lt;string array&gt;AssertReporter.GetAssertFailures()</code></pre><p>Returns an array of strings containing the assert failures stored in the <code>AssertReporter</code>.</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">&lt;integer&gt;AssertReporter.GetAssertFailuresCount()</code></pre><p>Returns the count of the assert failures stored in the <code>AssertReporter</code>. This is faster than calling <code>(AssertReporter.GetAssertFailuresCount()).count</code></p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">&lt;string array&gt;AssertReporter.GetExceptionFailures()</code></pre><p>Returns an array of strings containing any exceptions logged with the <code>AssertReporter</code>.</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">&lt;integer&gt;AssertReporter.GetExceptionFailuresCount()</code></pre><p>Returns the count of the exception failures stored in the <code>AssertReporter</code>. This is faster than calling <code>(AsserReporter.GetExceptionFailuresCount()).count</code></p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">&lt;array of strings&gt;AssertReporter.GetMessages()</code></pre><p>Returns an array of strings containing any messages that were logged through the <code>AssertReporter</code>, regardless of their type.</p>\n\
<p><strong>Logging Methods:</strong></p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">&lt;void&gt;AssertReporter.LogMessage &lt;string&gt;Message</code></pre><p>Saves a message with the <code>AssertReporter</code>. The messages can be retrieved later using <code>AssertReporter.GetMessages()</code></p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">&lt;void&gt;AssertReporter.LogException &lt;string&gt;Message</code></pre><p>Saves a message as an exception with the <code>AssertReporter</code>. The exception messages can be retrieved later using <code>AssertReporter.GetExceptionFailures()</code></p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">&lt;void&gt;AssertReporter.LogAssertFailure &lt;string&gt;Message</code></pre><p>Saves a message as an assert failure with the <code>AssertReporter</code>. The failure messages can be retrieved later using <code>AssertReporter.GetAssertFailures()</code></p>\n\
<p><strong>State Methods:</strong></p>\n\
<p>The following two methods can be used for saving some sort of state during test runs, and is only provided for future flexibility.</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">&lt;void&gt;AssertReporter.SetUserData &lt;string&gt;UserData</code></pre><p>Saves a custom user defined string in the <code>AssertReporter</code>.</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">&lt;void&gt;AssertReporter.GetUserData &lt;value&gt;argument</code></pre><p>Gets a custom user defined string from the <code>AssertReporter</code>.</p>\n\
<blockquote>\n\
<p>   <strong>EXAMPLE:</strong></p>\n\
<p>   The following example demonstrates the functionality of the assert functions and the AssertReporter in an artificial test situation where a number of variable have been defined in the local scope of a function and each variable is tested by an assert to verify its value (and of course most of these tests were designed to produce actual assert messages!)</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">   fn TestAsserts =\n\
   (\n\
       local undefinedValue\n\
       local booleanTrueVariable = true\n\
       local booleanFalseVariable = false\n\
       local floatVariable = 1.0\n\
       local stringVariable = &quot;MAXScript&quot;\n\
       local point3Variable = [1,2,3]\n\
       local matrix3Variable = matrix3 [1,0,0] [0,1,0] [0,0,1] [100,200,-300]\n\
   --this is line 10\n\
       AssertReporter.Clear()    \n\
       AssertReporter.LogMessage &quot;Asserts Demo Start...&quot;    \n\
       AssertReporter.SetUserData (&quot;Tested Values Were:&quot;+ undefinedValue as string + &quot;,&quot; + booleanTrueVariable as string + &quot;,&quot; + booleanFalseVariable as string + &quot;,&quot; + floatVariable as string + &quot;,&quot;+stringVariable+&quot;,&quot;+point3Variable as string + &quot;,&quot;+matrix3Variable as string)\n\
       assert_true booleanFalseVariable\n\
       assert_false booleanTrueVariable\n\
       assert_equal 2.0 floatVariable \n\
       assert_not_equal 2.0 floatVariable\n\
       assert_defined undefinedValue\n\
       assert_undefined floatVariable\n\
       assert_float 1.1 floatVariable tolerance:0.01\n\
       assert_float 1.1 floatVariable tolerance:0.2\n\
       assert_string_equal &quot;Python&quot; stringVariable\n\
       assert_string_equal &quot;maxscript&quot; stringVariable ignorecase:false message:(&quot; MAXScript is usually case-insensitive, but this assert isn&apos;t!&quot;)\n\
       assert_point3_equal [1.0,2.0,3.1] point3Variable\n\
       assert_point3_equal [1.0,2.0,3.1] point3Variable tolerance:0.1\n\
       assert_matrix_equal (matrix3 1) matrix3Variable\n\
       if floatVariable &gt; 0.0 do AssertReporter.LogAssertFailure &quot;FloatVariable seems to be positive, I am upset!&quot;\n\
       AssertReporter.LogException &quot;Nothing Really Worked!&quot;\n\
       AssertReporter.LogMessage &quot;Asserts Demo End.&quot;    \n\
   --this is line 30\n\
       format &quot;--All Logged Messages:\\n&quot; \n\
       print (AssertReporter.GetMessages())\n\
       format &quot;--User Data:\\n&quot; \n\
       print (AssertReporter.GetUserData 0)    \n\
       format &quot;--Failures: %\\n&quot; (AssertReporter.GetAssertFailuresCount()) \n\
       print (AssertReporter.GetAssertFailures())\n\
       format &quot;--Exceptions: %\\n&quot; (AssertReporter.GetExceptionFailuresCount()) \n\
       print (AssertReporter.GetExceptionFailures())\n\
   )\n\
\n\
   TestAsserts()\n\
</code></pre><p>   After saving the above code to a file called &quot;c:\\temp\\TestAsserts.ms&quot;, evaluating the script produced the following output:</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">   TestAsserts()\n\
   --All Logged Messages:\n\
   &quot;Asserts Demo Start...&quot;\n\
   &quot;Assert - Expected true, got false - (C:\\Temp\\TestAsserts.ms) in frame TestAsserts, at line 14&quot;\n\
   &quot;Assert - Expected false, got true - (C:\\Temp\\TestAsserts.ms) in frame TestAsserts, at line 15&quot;\n\
   &quot;Assert - Expected 2.0, got 1.0 - (C:\\Temp\\TestAsserts.ms) in frame TestAsserts, at line 16&quot;\n\
   &quot;Assert - Expected &quot;defined&quot;, got undefined - (C:\\Temp\\TestAsserts.ms) in frame TestAsserts, at line 18&quot;\n\
   &quot;Assert - Expected undefined, got 1.0 - (C:\\Temp\\TestAsserts.ms) in frame TestAsserts, at line 19&quot;\n\
   &quot;Assert - Expected 1.1, got 1.0 - (C:\\Temp\\TestAsserts.ms) in frame TestAsserts, at line 20&quot;\n\
   &quot;Assert - Expected &quot;Python&quot;, got &quot;MAXScript&quot; - (C:\\Temp\\TestAsserts.ms) in frame TestAsserts, at line 22&quot;\n\
   &quot;Assert - MAXScript is usually case-insensitive, but this assert isn&apos;t! - Expected &quot;maxscript&quot;, got &quot;MAXScript&quot; (C:\\Temp\\TestAsserts.ms) in frame TestAsserts, at line 23&quot;\n\
   &quot;Assert - Expected [1,2,3.1], got [1,2,3] - (C:\\Temp\\TestAsserts.ms) in frame TestAsserts, at line 24&quot;\n\
   &quot;Assert - Expected (matrix3 [1,0,0] [0,1,0] [0,0,1] [0,0,0]), got (matrix3 [1,0,0] [0,1,0] [0,0,1] [100,200,-300]) - (C:\\Temp\\TestAsserts.ms) in frame TestAsserts, at line 26&quot;\n\
   &quot;FloatVariable seems to be positive, I am upset!&quot;\n\
   &quot;Nothing Really Worked!&quot;\n\
   &quot;Asserts Demo End.&quot;\n\
   --User Data:\n\
   &quot;Tested Values Were:undefined,true,false,1.0,MAXScript,[1,2,3],(matrix3 [1,0,0] [0,1,0] [0,0,1] [100,200,-300])&quot;\n\
   --Failures: 11\n\
   &quot;Assert - Expected true, got false - (C:\\Temp\\TestAsserts.ms) in frame TestAsserts, at line 14&quot;\n\
   &quot;Assert - Expected false, got true - (C:\\Temp\\TestAsserts.ms) in frame TestAsserts, at line 15&quot;\n\
   &quot;Assert - Expected 2.0, got 1.0 - (C:\\Temp\\TestAsserts.ms) in frame TestAsserts, at line 16&quot;\n\
   &quot;Assert - Expected &quot;defined&quot;, got undefined - (C:\\Temp\\TestAsserts.ms) in frame TestAsserts, at line 18&quot;\n\
   &quot;Assert - Expected undefined, got 1.0 - (C:\\Temp\\TestAsserts.ms) in frame TestAsserts, at line 19&quot;\n\
   &quot;Assert - Expected 1.1, got 1.0 - (C:\\Temp\\TestAsserts.ms) in frame TestAsserts, at line 20&quot;\n\
   &quot;Assert - Expected &quot;Python&quot;, got &quot;MAXScript&quot; - (C:\\Temp\\TestAsserts.ms) in frame TestAsserts, at line 22&quot;\n\
   &quot;Assert - MAXScript is usually case-insensitive, but this assert isn&apos;t! - Expected &quot;maxscript&quot;, got &quot;MAXScript&quot; (C:\\Temp\\TestAsserts.ms) in frame TestAsserts, at line 23&quot;\n\
   &quot;Assert - Expected [1,2,3.1], got [1,2,3] - (C:\\Temp\\TestAsserts.ms) in frame TestAsserts, at line 24&quot;\n\
   &quot;Assert - Expected (matrix3 [1,0,0] [0,1,0] [0,0,1] [0,0,0]), got (matrix3 [1,0,0] [0,1,0] [0,0,1] [100,200,-300]) - (C:\\Temp\\TestAsserts.ms) in frame TestAsserts, at line 26&quot;\n\
   &quot;FloatVariable seems to be positive, I am upset!&quot;\n\
   --Exceptions: 1\n\
   &quot;Nothing Really Worked!&quot;\n\
   OK</code></pre><p>   Note that the Assert on line 21 did not output a failure because the tolerance was set to 0.2, allowing 1.0 to pass a test against 1.1, which the Assert on the previous line 20 did not allow.</p>\n\
<p>   Line 23 had a custom message prepended to the default message.</p>\n\
<p>   Line 25 also passed due to a higher tolerance delta.</p>\n\
<p>   As you can see, with these assert functions you can collect data about the state of variables with various value types or the return values of expressions and collect the information in the AssertReporter to analyze after a test run of the code or dump to a log file for later processing.</p>\n\
</blockquote>\n\
\n\
  </div>\n\
\n\
    <div class=\"related-links\">\n\
  \n\
    <div class=\"familylinks\">\n\
      <div class=\"parentLink\">\n\
        <strong>Parent page:</strong> <a class=\"link\" href=\"../The-MAXScript-Debugger.html\">The MAXScript Debugger</a>\n\
      </div>\n\
  \n\
    </div>\n\
  \n\
      <div class=\"relinfo\">\n\
        <h4 class=\"related\">Related pages</h4>\n\
        <ul>\n\
          <li><a class=\"link\" href=\"../The-MAXScript-Debugger.html\">The MAXScript Debugger</a></li>\n\
          <li><a class=\"link\" href=\"../../3ds-Max-Objects-and-Interfaces/Interfaces/Core-Interfaces/Core-Interfaces-Documentation/M/Interface-MXSDebugger.html\">Interface: MXSDebugger</a></li>\n\
        </ul>\n\
      </div>\n\
    </div>\n\
\n\
</body>\n\
</html>\n\
";