var topic = "<html>\n\
<head>\n\
	<script src=\"../scripts/lib/jquery-1.11.1.min.js\" type=\"text/javascript\"/></script\n\
	<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\">\n\
	<!--script type=\"text/javascript\" src=\"..\\../scripts/utils/adsk.redirect.js\"/></script>	\n\
	<script type=\"text/javascript\" src=\"..\\../scripts/prettify/lang-maxscript.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\../scripts/prettify/run_prettify.js\"/></script-->\n\
    <title>MAXScript Help</title>\n\
    <meta name=\"topicid\" content=\"GUID-6E1C44F4-81C0-400F-9558-E565E1F77D36\" />\n\
    </head>\n\
<body height=\"100%\">\n\
  <a name=\"top\"></a>\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2022/ENU/MAXScript-Help/styles/bootstrap.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2022/ENU/MAXScript-Help/styles/additions.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2022/ENU/MAXScript-Help/styles/font-awesome.min.css\">\n\
      <script src=\"/cloudhelp/2022/ENU/MAXScript-Help/scripts/prettify/lang-maxscript.js\"></script>\n\
\n\
  <div class=\"body_content\" id=\"body-content\">\n\
  <div class=\"head-block\"><h1>By Reference Parameter Passing</h1></div><p>MAXScript supports by-reference parameter passing. This allows a reference to be passed to a variable, property, or array element in the caller&apos;s context as a parameter to a function and any assignments to that parameter in the called function is assigned directly to the caller&apos;s variable, property, or array element. This is often used as a mechanism for passing multiple results back from a function.</p>\n\
<p>To define by-reference parameters for a function, you need to prefix them with an &apos;<strong>&amp;</strong>&apos; character.</p>\n\
<p><strong>System Exception Note:</strong></p>\n\
<p>The following example is an error:</p>\n\
<blockquote>\n\
<p>   <strong>ERROR EXAMPLE</strong></p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">   fn test1 &amp;p3 = return &amp;p3.x\n\
   p=[1,2,3]\n\
   r=test1 p\n\
   *r\n\
</code></pre></blockquote>\n\
<p><strong>Result: access violation!</strong></p>\n\
<p>It is <strong>an error</strong> to pass back a reference to a local that has gone from the call stack, but it is very difficult to detect this efficiently. Be careful <code>not to pass back &amp;refs</code> to local variables as it might cause system exceptions.</p>\n\
<blockquote>\n\
<p>   <strong>EXAMPLE</strong></p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">   function foo x y &amp;z =\n\
   (\n\
   -- ...\n\
   z = x * y\n\
   -- ...\n\
   return true\n\
   )\n\
   -- test function\n\
   z = 12\n\
   foo 5 6 &amp;z\n\
</code></pre><p>   Declares a function taking three parameters, x, y and z. The <code>&amp;z</code> in the parameter list indicates the z parameter is to be passed by-reference. To call this function, you provide a reference value for the z parameter using the new &apos;<strong>&amp;</strong>&apos; reference operator.</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">   local var1, var2\n\
   ...\n\
   var1 = foo 2 3 &amp;var2\n\
</code></pre><p>   The above example calls the foo function with the arguments 2, 3 and <code>&amp;var2</code> . The <code>&amp;var2</code> is a reference to the var2 local variable. When the function is called, the z parameter in the body is a reference to <code>var2</code> local in the caller, and the</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">   z = x * y\n\
</code></pre><p>   in the body of foo will cause x * y to be assigned to the <code>var2</code> local in the caller. After</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">   var1 = foo 2 3 &amp;var2\n\
</code></pre><p>   <code>var1</code> will contain <code>true</code> and <code>var2</code> will contain <code>x * y</code> or <code>6</code> .</p>\n\
</blockquote>\n\
<p>Any number of by-reference parameters can be used and they can be keyword parameters if needed:</p>\n\
<blockquote>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">   fn foo x y &amp;z: = ...\n\
</code></pre><p>   called like this:</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">   var1 = foo 2 3 z:&amp;var2 </code></pre></blockquote>\n\
<p>The &apos; <code>&amp;</code> &apos; reference operator used to supply references in a call can be applied to any <code>&lt;accessor&gt;</code> construct in MAXScript. These are:</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">variables (locals, globals, rollout locals, plug-in locals, etc.)\n\
property access\n\
array index\n\
parameter (of a scripted plug-in)\n\
</code></pre><blockquote>\n\
<p>   <strong>EXAMPLE</strong></p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">   p = [1, 2, 3]\n\
   foo 2 3 &amp;p.x\n\
</code></pre><p>   would cause the x coordinate in the point to be passed by-reference and so after the call, the value in p would be</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">   [6, 2, 3]\n\
</code></pre></blockquote>\n\
<blockquote>\n\
<p>   <strong>EXAMPLE</strong></p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">   a = #(1, 2, 3, 4, 5)\n\
   foo 2 3 &amp;a[3]\n\
</code></pre><p>   would cause the 3rd element in the array to be passed by-reference and so after the call, the value in a would be #(1, 2, 6, 4, 5)</p>\n\
</blockquote>\n\
<div class=\"note note:\"><span class=\"notetitle\">Note:</span> </div>\n\
<p>This mechanism is used by the Function Publishing interface in MAXScript to support BY-REF interface method parameters and results. Any parameter defined as by-ref (or with a type code suffix _BR) in the published interface, requires an &apos;&amp;&apos; reference value.</p>\n\
\n\
  </div>\n\
\n\
    <div class=\"related-links\">\n\
  \n\
    <div class=\"familylinks\">\n\
      <div class=\"parentLink\">\n\
        <strong>Parent page:</strong> <a class=\"link\" href=\"../Creating-Functions.html\">Creating Functions</a>\n\
      </div>\n\
  \n\
    </div>\n\
  \n\
      <div class=\"relinfo\">\n\
        <h4 class=\"related\">Related pages</h4>\n\
        <ul>\n\
          <li><a class=\"link\" href=\"Dereferencing-Operator.html\">Dereferencing Operator</a></li>\n\
          <li><a class=\"link\" href=\"Visible-Class-For-Reference.html\">Visible Class For &#x27;&amp;&#x27; Reference Values</a></li>\n\
        </ul>\n\
      </div>\n\
    </div>\n\
\n\
</body>\n\
</html>\n\
";