var topic = "<html>\n\
<head>\n\
	<script src=\"../scripts/lib/jquery-1.11.1.min.js\" type=\"text/javascript\"/></script\n\
	<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\">\n\
	<!--script type=\"text/javascript\" src=\"..\\../scripts/utils/adsk.redirect.js\"/></script>	\n\
	<script type=\"text/javascript\" src=\"..\\../scripts/prettify/lang-maxscript.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\../scripts/prettify/run_prettify.js\"/></script-->\n\
    <title>MAXScript Help</title>\n\
    <meta name=\"topicid\" content=\"GUID-5FBA4F99-5A57-4B36-9008-F8EC68340AA6\" />\n\
    </head>\n\
<body height=\"100%\">\n\
  <a name=\"top\"></a>\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2022/ENU/MAXScript-Help/styles/bootstrap.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2022/ENU/MAXScript-Help/styles/additions.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2022/ENU/MAXScript-Help/styles/font-awesome.min.css\">\n\
      <script src=\"/cloudhelp/2022/ENU/MAXScript-Help/scripts/prettify/lang-maxscript.js\"></script>\n\
\n\
  <div class=\"body_content\" id=\"body-content\">\n\
  <div class=\"head-block\"><h1>Structure Definition</h1></div><p>You can create your own compound values in MAXScript using <strong>Structure definitions</strong>.</p>\n\
<p>Structure definitions let you define the layout of new &apos;classes&apos; of values that you can then create and work with in your code.</p>\n\
<p>The <strong>syntax</strong> for a structure definition is:</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">struct &lt;struct_name&gt; ( &lt;member&gt; { , &lt;member&gt; } )</code></pre><p>where each <code>&lt;member&gt;</code> can be one of:</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">&lt;name&gt; [ = &lt;expr&gt; ] --name and optional initial value\n\
&lt;function_def&gt;\n\
</code></pre><blockquote>\n\
<p>   <strong>EXAMPLE</strong></p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">   struct person (name, height, age, sex)\n\
</code></pre></blockquote>\n\
<p>The above example defines a new &apos;person&apos; class. You can create values of this class using the &apos;person&apos; constructor:</p>\n\
<blockquote>\n\
<p>   <strong>EXAMPLE</strong></p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">   bill = person name: &quot;Bill&quot; height:72 age:34 sex:#male\n\
</code></pre></blockquote>\n\
<p>This creates an instance of the <code>person</code> class, storing the instance in variable <code>bill</code> . Member <code>name</code> is initialized to the string value &quot;Bill&quot;, <code>height</code> to the integer value 72, <code>age</code> to the integer value 34, and <code>sex</code> to the name value #male.</p>\n\
<blockquote>\n\
<p>   <strong>EXAMPLE</strong></p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">   joe = person name: &quot;Joseph&quot; sex:#male\n\
</code></pre></blockquote>\n\
<p>This creates an instance of the <code>person</code> class, storing the instance in variable <code>joe</code> . Member <code>name</code> is initialized to the string value &quot;Joseph&quot; and <code>sex</code> to the name value #male. As the <code>height</code> and <code>age</code> members are not assigned values and do not have optional initial values supplied in the structure definition, they default to a value of <code>undefined</code> .</p>\n\
<p>You can access structured values using the standard property accessing syntax in MAXScript.</p>\n\
<blockquote>\n\
<p>   <strong>EXAMPLE</strong></p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">   bill.age  --returns 34\n\
   joe.age  --returns undefined\n\
   joe.age = bill.age-4 --assigns Bill&apos;s age less 4 to Joe&apos;s age.\n\
</code></pre></blockquote>\n\
<p>Structure definitions are useful as an alternative to arrays when you have a fixed, usually small number of independent components and the code to work with them is much clearer when they can be referenced by property name, rather than by index number.</p>\n\
<p>As with function definitions, a structure definition actually creates a value to represent the definition and stores it in a variable of the same name as the structure. You can store the structure definitions in other compound objects or pass them as function arguments. The <code>classOf()</code> function returns this structure definition value when applied to these values, so you can use it to test the definition of structure instances at runtime.</p>\n\
<blockquote>\n\
<p>   <strong>EXAMPLE</strong></p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">   classOf bill --returns person\n\
</code></pre></blockquote>\n\
<p>The <code>struct</code> value constructors are just like function calls and take positional argument initializers as well as keyword initializers. The elements of the new <code>struct</code> are filled in order from any positional arguments and then by name from any keyword arguments.</p>\n\
<blockquote>\n\
<p>   <strong>FOR EXAMPLE,</strong></p>\n\
<p>   given the following <code>struct</code> definition:</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">   struct foo (name, age, height, hair=&quot;brown&quot;)\n\
</code></pre><p>   you can create instances in several ways:</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">   f1 = foo &quot;bill&quot; 23 72 --fills in name, age, height in order\n\
   f2 = foo age:45 name: &quot;sam&quot; --random named initializers\n\
   f3 = foo &quot;mary&quot; hair: &quot;red&quot; --first name, then keywords\n\
</code></pre></blockquote>\n\
<p>There are several methods associated with <code>struct</code> values:</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">getProperty &lt;struct&gt; &lt;propname&gt;</code></pre><p>Allows you to access properties of the structure.</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">getPropNames &lt;struct&gt;</code></pre><p>Returns the properties of the structure as an array.</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">copy &lt;struct&gt;</code></pre><p>Creates a copy of the structure. The copy made is called a <em>shallow copy</em>. Only a copy of the upper-level value itself (that is, the <code>struct</code> value) is created. Copies are not made of compound values stored in the structure, rather the same compound value is stored in both the <code>struct</code> values. This can be seen in the following example, where changing a component value of a compound value in a copy of a <code>struct</code> value also changes the value in the original <code>struct</code> value:</p>\n\
<blockquote>\n\
<p>   <strong>SCRIPT:</strong></p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">   Struct test (pos1, pos2) --define a structure\n\
   testval = test [1,2,3] [4,5,6] --create struct value\n\
   testval_copy=copy testval --copy the struct value\n\
   testval_copy.pos1.x=10 --change component value of a compound\n\
   --value in the copy\n\
   testval --look at original struct value\n\
</code></pre><p>   <strong>OUTPUT:</strong></p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">   #Struct:test( --result line 1\n\
     pos1:&lt;data&gt;; Public,\n\
     pos2:&lt;data&gt;; Public)\n\
   test(pos1:[1,2,3] pos2:[4,5,6]) --result line 2\n\
   test(pos1:[1,2,3] pos2:[4,5,6]) --result line 3\n\
   10 --result line 4\n\
   (test pos1:[10,2,3] pos2:[4,5,6]) --result line 6\n\
</code></pre></blockquote>\n\
\n\
  </div>\n\
\n\
    <div class=\"related-links\">\n\
  \n\
    <div class=\"familylinks\">\n\
      <div class=\"parentLink\">\n\
        <strong>Parent page:</strong> <a class=\"link\" href=\"../Structure-Definition.html\">Structure Definition</a>\n\
      </div>\n\
  \n\
    </div>\n\
  \n\
      <div class=\"relinfo\">\n\
        <h4 class=\"related\">Related pages</h4>\n\
        <ul>\n\
          <li><a class=\"link\" href=\"Defining-Local-Functions-in.html\">Defining Local Functions in Structures</a></li>\n\
          <li><a class=\"link\" href=\"Structure-Inherited-Methods.html\">Structure Inherited Methods</a></li>\n\
          <li><a class=\"link\" href=\"Structure-Scope-While.html\">Structure Scope While Initializing</a></li>\n\
          <li><a class=\"link\" href=\"Structure-Creation-and-Cloning.html\">Structure Creation and Cloning Event Handlers</a></li>\n\
          <li><a class=\"link\" href=\"Private-and-Public-Members-in.html\">Private and Public Members in Structure Definitions</a></li>\n\
        </ul>\n\
      </div>\n\
    </div>\n\
\n\
</body>\n\
</html>\n\
";