var topic = "<html>\n\
<head>\n\
	<script src=\"../scripts/lib/jquery-1.11.1.min.js\" type=\"text/javascript\"/></script\n\
	<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\">\n\
	<!--script type=\"text/javascript\" src=\"..\\../scripts/utils/adsk.redirect.js\"/></script>	\n\
	<script type=\"text/javascript\" src=\"..\\../scripts/prettify/lang-maxscript.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\../scripts/prettify/run_prettify.js\"/></script-->\n\
    <title>MAXScript Help</title>\n\
    <meta name=\"topicid\" content=\"GUID-513285B3-DBF6-471E-B587-B5BE14D4D875\" />\n\
    </head>\n\
<body height=\"100%\">\n\
  <a name=\"top\"></a>\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2022/ENU/MAXScript-Help/styles/bootstrap.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2022/ENU/MAXScript-Help/styles/additions.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2022/ENU/MAXScript-Help/styles/font-awesome.min.css\">\n\
      <script src=\"/cloudhelp/2022/ENU/MAXScript-Help/scripts/prettify/lang-maxscript.js\"></script>\n\
\n\
  <div class=\"body_content\" id=\"body-content\">\n\
  <div class=\"head-block\"><h1>Change Handlers and When Constructs</h1></div><p>Change Handlers are used to detect when certain types of user events are performed on objects in a scene, allowing you to write scripts that respond to user operations like object moves, vertex edits, object deletions, name changes, and so on. Change Handlers are applied to individual MAXWrapper objects and specify an attribute to monitor for that object and an expression to evaluate when that attribute changes. Example attributes that can be monitored are geometry, name, transform, and parameters. Change Handlers are not stored in a <strong>3ds Max</strong> scene.</p>\n\
<p>The <code>ChangeHandler : Value</code> class instances represent change handler setups. ChangeHandler values are created using the <code>when</code> construct. Each time a <code>when</code> construct is executed, it creates and returns a new ChangeHandler instance. You must store this ChangeHandler value in a variable or array so that you can dismiss the handler when it is appropriate to do so.</p>\n\
<p>The <code>when</code> construct defines a change handler function for a certain type of event on one or more objects. The system then automatically calls this function whenever the event occurs.</p>\n\
<div class=\"note note:\"><span class=\"notetitle\">Note:</span> Change events are only generated when the user changes the object. Events are not generated if the change is triggered by an animation.</div>\n\
<p>The syntax of the <code>when</code> construct has two forms as follows:</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">when &lt;attribute&gt; &lt;objects&gt; change[s] [ id:&lt;name&gt; ] \\ \n\
[handleAt: #redrawViews|#timeChange] \\ \n\
[ &lt;object_parameter&gt; ] do &lt;expr&gt;</code></pre><pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">when &lt;objects&gt; deleted [ id:&lt;name&gt; ] \\ \n\
[handleAt:#redrawViews|#timeChange] \\\n\
[ &lt;object_parameter&gt; ] do &lt;expr&gt; </code></pre><p>In the first form, the <code>&lt;attribute&gt;</code> can be one of:</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\"> topology geometry name[s] transform select parameters subAnimStructure controller children</code></pre><p>These specify the attribute of the given object(s) to be tracked for change (see below for more details).</p>\n\
<p>The second form tracks object deletion by the user or other scripts. This is typically used if you have tables or other structures containing MAXScript object values and you want to remove deleted objects from them as the user modifies the scene.</p>\n\
<blockquote>\n\
<p>   <strong>NOTES:</strong></p>\n\
<p>   The <code>#selectedNodesPreDelete</code></p>\n\
<p>   <a href=\"General-Event-Callback-Mechanism/Node-Related-Notifications.html\"\">general callback</a></p>\n\
<p>   is the far superior method to use to catch node deletion by the user. It is called before <strong>anything</strong> happens to the nodes and will report the deletion of any node. In contrast, the <code>when</code> construct operates only on nodes explicitly registered with the callback.</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">   mypot1 = teapot() --create a teapot\n\
   fn whendeleted = --define a callback function\n\
   (\n\
   local deletedobjects = callbacks.notificationParam()\n\
   format &quot;Callback: %\\n&quot; deletedobjects\n\
   -- This will printevery object that was deleted.\n\
   -- By this point, the object isstillattached to it&apos;s parents\n\
   -- and is still in groups etc...\n\
   -- Nothing has been done to the nodeyet.\n\
   )\n\
   when mypot1 deleted id:#foo obj do --define a when construct\n\
   (\n\
   format &quot;When Construct:%\\n&quot; obj\n\
   )\n\
   --unregisterany preDeletecallbacks\n\
   callbacks.removeScripts #selectedNodesPreDelete\n\
   --register the function as general callback\n\
   callbacks.addscript #selectedNodesPreDelete &quot;whendeleted()&quot;\n\
</code></pre><p>   A regression in <strong>3ds Max 9</strong> caused a change in the way the <code>when</code> construct was called. In releases prior to <strong>3ds Max 9</strong>, the <code>when</code> construct was called before the actual deletion of the object. In <strong>3ds Max 9</strong>, the <strong>when</strong> callback in the above code would return <deleted scene node> instead.</deleted></p>\n\
<p>   This behavior has been fixed in <strong>3ds Max 2008</strong>.</p>\n\
</blockquote>\n\
<p>In both forms, the <code>&lt;objects&gt;</code> argument can be one of the following:</p>\n\
<ul>\n\
<li><p>a single <strong>3ds Max</strong> object such as, a scene node, controller, modifier, material, and so on.</p>\n\
</li>\n\
<li><p>one of the object sets such as, &apos;selection&apos; or &apos;cameras&apos;.</p>\n\
</li>\n\
<li><p>a wild-card pathname like $foo*.</p>\n\
</li>\n\
<li><p>an array of <strong>3ds Max</strong> objects.</p>\n\
</li>\n\
</ul>\n\
<p>If more than one object is specified, the handler is called each time the given attribute of any of the objects is signaled as changed by the <strong>3ds Max</strong> core.</p>\n\
<p>The optional <code>id:</code> parameter specifies an ID for one or more handlers as a name literal. The ID name can later be used to delete the handler and if the same name is given to several handlers, they can be deleted as a group.</p>\n\
<p>The optional <code>handleAt:</code> parameter signals that the change handler expression must <strong>not</strong> be executed immediately upon notification, but delayed until either the <strong>3ds Max</strong> viewports are redrawn ( <code>#redrawViews</code> ) or current <strong>3ds Max</strong> animation time is changed ( <code>#timeChange</code> ). Delaying the processing of the change handler expression is highly recommended, as described in the Caution section below. An example usage of the <code>handleAt:</code> parameter is:when select $ changes id:#foo handleAt:#redrawViews do ...</p>\n\
<p>The optional <code>&lt;object_parameter&gt;</code> lets you specify the name of a parameter variable that will be accessible in the do <code>&lt;expr&gt;</code> and will hold the actual object that has changed. You typically specify this parameter name if the handler will respond to changes in many objects, so you can determine which one has changed. The <code>&lt;expr&gt;</code> can be a single expression or block expression.</p>\n\
<blockquote>\n\
<p>   <strong>EXAMPLE</strong></p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">   b1 = box()\n\
   b2 = box()\n\
   b3 = box()\n\
   when transform $box001 changes do\n\
   $box002.pos = $box001.pos + delta\n\
   max select all\n\
   when names selection change obj do update_name_table obj\n\
   when #($box001, $box002, $box003) deleted obj do\n\
   (\n\
   messageBox &quot;Warning!&quot;\n\
   deleteItem obj_table (findItem obj_table obj)\n\
   )\n\
</code></pre></blockquote>\n\
<p>The change attributes are interpreted as follows:</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">topology</code></pre><p>Signaled when the topology of an object changes in the Modify panel such as, using a mesh smooth, optimize, or vertex delete.</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">geometry</code></pre><p>Signaled when the geometry of an object changes such as, by moving a vertex or using an animated modifier.</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">name or names</code></pre><p>Signaled when the name of an object is changed if this occurs because a user edits the name in one of the <strong>3ds Max</strong> command panels. The handler is called repeatedly with each character that is changed.</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">transform</code></pre><p>Signaled when the transform of an object is changed such as, by a move, rotate, or scale.</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">select</code></pre><p>Signaled when a scene node moves into or out of the current selection set. You should interrogate the <code>&lt;node&gt;.isSelected</code> property to determine the new state.</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">parameters</code></pre><p>Signaled when any parameters are changed in the object. This is something of a catch all because the core signals this event in many situations.</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">subAnimStructure</code></pre><p>Signaled when the dynamic subAnim structure changes such as, when a new vertex becomes animated in an editable mesh, or when a new controller is added to a list controller. Also called when subAnims are reassigned, for example, when a material is changed in an object.</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">controller</code></pre><p>Signaled when a new controller is assigned to one of the object&apos;s tracks.</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">children</code></pre><p>Signaled when an object has immediate children added or removed.</p>\n\
<h2 id=\"deleting-change-handlers\">Deleting Change Handlers</h2><p>You can use the following two methods for deleting change handlers:</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">deleteChangeHandler &lt;change_handler&gt;</code></pre><p>deletes specified change handler . <code>change_handler</code> is the value returned by a <code>when</code> construct.</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">deleteAllChangeHandlers [ id:&lt;name&gt; ]</code></pre><p>If optional <code>id:</code> parameter is not specified, deletes all change handlers. If optional <code>id:</code> parameter is specified, deletes all change handlers with the specified id.</p>\n\
<blockquote>\n\
<p>   <strong>FOR EXAMPLE:</strong></p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">   deleteAllChangeHandlers id:#foo\n\
</code></pre></blockquote>\n\
<p>For efficiency reasons, you do not want irrelevant change handlers running in the background, so it is essential that you delete those that you no longer need.</p>\n\
<p><strong>Special Considerations</strong></p>\n\
<ul>\n\
<li><p>If a runtime error occurs in the body of a change handler while it is being executed, an error message is displayed and that handler is permanently disabled.</p>\n\
</li>\n\
<li><p>If all of the objects being tracked by a change handler are deleted, the change handler is also deleted.</p>\n\
</li>\n\
<li><p>The <code>do &lt;expr&gt;</code> change handler code is executed in a special context and not in the context of the code that created it. This means that the <code>&lt;expr&gt;</code> code cannot contain references to local variables in outer code nestings surrounding the <code>when</code> because those variables are on an execution stack that does not exist at the time the <code>when</code> handler is called. The compiler detects any illegal references to outer locals and generates a compiler message. An important exception to this is utility and rollout panel locals such as, local functions, rollout variables, and nested rollouts. You can refer to them in change handler code inside rollout code as they are associated directly with their rollout or utility object.</p>\n\
</li>\n\
<li><p>Change handlers are called only for user initiated events, and not for changes resulting from a change in controller values. For example, a change handler on the transform attribute of a node would be called when the user moves the node. If the position of the node is animated, and you play back the animation, the transform attribute change handler is not called.</p>\n\
</li>\n\
<li><p>If you assign a change handler to an attribute on multiple objects, the change handler is called once per object if that object&#x2019;s attribute changes.</p>\n\
</li>\n\
</ul>\n\
<blockquote>\n\
<p>   <strong>FOR EXAMPLE, IF YOU SAY:</strong></p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">   when select $ changes obj do update_modifier_list obj\n\
</code></pre><p>   The function <code>update_modifier_list</code> is called whenever any object is selected or deselected. This is true even if you do an Edit / Select All. In this case, <code>update_modifier_list</code> will be called once for each object currently selected (as the objects are deselected), and then once for all objects in the scene (as the objects are selected). If <code>update_modifier_list</code> is at all processor-intensive, a significant system slowdown can occur.</p>\n\
</blockquote>\n\
<ul>\n\
<li><p>Change handlers are only applied to object already present in the scene. Change handlers are not automatically applied to newly created objects.</p>\n\
</li>\n\
<li><p>If you have multiple change handlers defined, the order in which the change handlers are called is somewhat arbitrary.</p>\n\
</li>\n\
<li><p>Due to the way that <strong>3ds Max</strong> internally processes notification signals, the <strong>$</strong> form of accessing selected objects is not recommended in a select change handler. To access the selected objects, you must use the <strong>selection</strong> objectset. This is because <strong>$</strong> relies on information that has not yet been set in the selection processing whereas <strong>selection</strong> uses a different method of accessing selections and the information it uses has been set up.</p>\n\
</li>\n\
</ul>\n\
<blockquote>\n\
<p>   <strong>CAUTION</strong> <strong>!</strong></p>\n\
<p>   The change handler system is based on <strong>3ds Max</strong>&apos;s internal notification system which essentially drives all animation and interactivity within <strong>3ds Max</strong>.</p>\n\
<p>   There are cases when the core sends many signals for the same change, so setting up change handlers on many objects that do vast amounts of computing can significantly slow down the system.</p>\n\
<p>   Strictly speaking, change handlers are supposed to be used to just set &quot;dirty flags&quot; in a lightweight, order-independent way, and then use Redraw Views or Time Change callbacks to actually cause the triggered processing.</p>\n\
<p>   You must attempt to use change handlers judiciously, and not, for example, as a simple way to get scripted controllers.</p>\n\
<p>   If you do find that a desired setup is being flooded with unnecessary signals, you must use the <code>handleAt:</code> to delay the actual processing of the event handler script until a redrawViews or timeChange event occurs.</p>\n\
</blockquote>\n\
\n\
  </div>\n\
\n\
    <div class=\"related-links\">\n\
  \n\
    <div class=\"familylinks\">\n\
      <div class=\"parentLink\">\n\
        <strong>Parent page:</strong> <a class=\"link\" href=\"../Change-Handlers-and-Callbacks.html\">Change Handlers and Callbacks</a>\n\
      </div>\n\
  \n\
    </div>\n\
  \n\
      <div class=\"relinfo\">\n\
        <h4 class=\"related\">Related pages</h4>\n\
        <ul>\n\
          <li><a class=\"link\" href=\"../Change-Handlers-and-Callbacks.html\">Change Handlers and Callbacks</a></li>\n\
          <li><a class=\"link\" href=\"Time-Change-Callback-Mechanism.html\">Time Change Callback Mechanism</a></li>\n\
          <li><a class=\"link\" href=\"Viewport-Redraw-Callback.html\">Viewport Redraw Callback Mechanism</a></li>\n\
          <li><a class=\"link\" href=\"General-Event-Callback-Mechanism.html\">General Event Callback Mechanism</a></li>\n\
        </ul>\n\
      </div>\n\
    </div>\n\
\n\
</body>\n\
</html>\n\
";