var topic = "<html>\n\
<head>\n\
	<script src=\"../scripts/lib/jquery-1.11.1.min.js\" type=\"text/javascript\"/></script\n\
	<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\">\n\
	<!--script type=\"text/javascript\" src=\"..\\..\\../scripts/utils/adsk.redirect.js\"/></script>	\n\
	<script type=\"text/javascript\" src=\"..\\..\\../scripts/prettify/lang-maxscript.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\..\\../scripts/prettify/run_prettify.js\"/></script-->\n\
    <title>MAXScript Help</title>\n\
    <meta name=\"topicid\" content=\"GUID-584BA3B0-ADBC-45A7-9BF4-ACE81930B2A0\" />\n\
    </head>\n\
<body height=\"100%\">\n\
  <a name=\"top\"></a>\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2022/ENU/MAXScript-Help/styles/bootstrap.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2022/ENU/MAXScript-Help/styles/additions.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2022/ENU/MAXScript-Help/styles/font-awesome.min.css\">\n\
      <script src=\"/cloudhelp/2022/ENU/MAXScript-Help/scripts/prettify/lang-maxscript.js\"></script>\n\
\n\
  <div class=\"body_content\" id=\"body-content\">\n\
  <div class=\"head-block\"><h1>RCMenu Clauses</h1></div><p>The <code>&lt;rcmenu_body&gt;</code> of a right-click menu definition is made up of a sequence of scripted right-click menu clauses as follows:</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">&lt;rcmenu_body&gt; ::= { &lt;rcmenu_clause&gt; }+</code></pre><p>Scripted right-click menu clauses define the components of a scripted right-click menu and can be one of three basic things:</p>\n\
<ul>\n\
<li><p><strong>Local variables, functions or structure definitions</strong> are variables, functions, and structures whose scope is the scripted right-click menu. These locals will exist as long as the scripted right-click menu value exists. The scripted right-click menu value will exist until the scripted right-click menu value is redefined or deleted. Local variables are particularly useful for storing working data associated with the right-click menu such as picked objects.</p>\n\
</li>\n\
<li><p><strong>User-interface items</strong> are displayed in the right-click menu, such as menu items, separators, and submenus.</p>\n\
</li>\n\
<li><p><strong>Event handlers</strong> are special kinds of functions that are associated with particular user-interface items. Event handlers specify the processing you want to occur when a user interacts with an user-interface item, for example pressing a menu item or opening a submenu. These user actions generate named <strong>events</strong> and the optional event handler you supply for that event is called when the user action occurs.</p>\n\
</li>\n\
</ul>\n\
<p>The visibility of locals, and accessing scripted right-click menu locals from external code, are the same as described for utility rollouts, and described in <a href=\"./../Scripted-Utilities-and-Rollouts/Visibility-of-Locals-Functions.html\">Visibility of Locals, Functions, Structures, and User-Interface Items in Rollout Code</a> and <a href=\"./../Scripted-Utilities-and-Rollouts/Accessing-Locals-and-Other-Items.html\">Accessing Locals and Other Items in a Rollout from External Code</a>. In both utility rollouts and scripted right-click menus, the name given to a user-interface item is used to name an automatically constructed local variable that will hold the value representing the item. It is also used to associate event handler functions with the item. One difference between the scooping of these variables is that they are local to the rollout in utility rollouts, and local to the scripted right-click menu in scripted right-click menus. This means that all user-interface items in a scripted right-click menu must have a unique name within the scripted right-click menu, even for those items present in a <code>subMenu</code> .</p>\n\
<p>Formally, the syntax of a <code>&lt;rcmenu_clause&gt;</code> is defined as follows:</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">&lt;rcmenu_clause&gt; ::= &lt;local_variable_decl&gt; | &lt;local_function_decl&gt; | &lt;local_struct_decl&gt; | &lt;user_interface_item&gt; | &lt;event_handler&gt;</code></pre><h2 id=\"locals\">Locals</h2><p>A <code>&lt;local_variable_decl&gt;</code> , <code>&lt;local_function_decl&gt;</code> , and <code>&lt;local_struct_decl&gt;</code> are exactly the same as local variable, function, and structure definitions in MAXScript:</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">&lt;local_variable_decl&gt; ::= local &lt;decl&gt; { , &lt;decl&gt; } \n\
&lt;decl&gt; ::= &lt;name&gt; [ = &lt;expr&gt; ] -- optional initial value</code></pre><pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">&lt;local_function_decl&gt; ::= [ mapped ](function | fn) &lt;name&gt; { &lt;argument&gt; } = &lt;expr&gt;</code></pre><pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">&lt;local_struct_decl&gt; ::= struct &lt;name&gt; ( &lt;member&gt; { , &lt;member&gt; })&lt;member&gt; ::= ( &lt;name&gt; [ = &lt;expr&gt; ] | &lt;local_function_decl&gt; )</code></pre><blockquote>\n\
<p>   <strong>EXAMPLES OF THE ABOVE (IN ORDER) ARE:</strong></p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">   local numSelected\n\
</code></pre><pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">   local numSelected = 0\n\
</code></pre><pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">   fn onlyOneSelected = selection.count == 1\n\
</code></pre><pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">   struct parents (mother=&quot;&quot;,father=&quot;&quot;)\n\
</code></pre></blockquote>\n\
<p>Global variables cannot be declared as a scripted right-click menu clause, however they can be declared within event-handler scripts. If you need to ensure that a variable name references a global variable, declare the variable name as global immediately before defining the right-click menu in your script.</p>\n\
<p>When writing scripts, it is good programming practice to explicitly declare your local and global variables. Implicit declaration is provided as a short-hand, typically used when working in the Listener interactively or developing short scripts. When developing extended scripts, explicitly declaring variables can reduce errors and improve readability of the code. It is also recommend that you declare as local all variables unless you really want them to be global variables. The reasons for this are described in <a href=\"./../../../MAXScript-Language-Reference/Variables-Assignment-and-Scope/Scope-of-Variables.html\">Scope of Variables</a>.</p>\n\
<h2 id=\"user-interface-items\">User-Interface Items</h2><p>A <code>&lt;user_interface_item&gt;</code> defines an individual menu item, separator, or submenu user-interface item that will appear in the right-click menu. These user-interface items are described in <a href=\"RCMenu-User-Interface-Items.html\">RCMenu User-Interface Items</a>.</p>\n\
<h2 id=\"event-handlers\">Event Handlers</h2><p>An <code>&lt;event_handler&gt;</code> is a special function definition local to a scripted right-click menu that you provide to handle the processing you want to occur when a user clicks on a menu item. This user action generates a named event and any event handler you supply for that event is called when the action occurs. The syntax for defining a handler is as follows:</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">on &lt;item_name&gt; &lt;event_name&gt; do &lt;expr&gt;</code></pre><p>The <code>&lt;item_name&gt;</code> specifies the name of the item for which this handler is connected. The <code>&lt;event_name&gt;</code> specifies the type of event to be handled.</p>\n\
<p>There is only one type of user-interface item event:</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">picked</code></pre><p>The <code>picked</code> handler expression is executed when a menu item is picked from the right-click menu. Any MAXScript action like creating a object, adding a modifier, and so on can be performed in this expression.</p>\n\
<p>In addition to the event handlers you specify for particular menu items in a right-click menu, you can define a handler function that is called when the entire right-click menu is first opened by the user. This event handler is useful for initialization code. The syntax for this event handler is as follows:</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-maxscript prettyprint\">on &lt;rcmenu&gt; open do &lt;expr&gt;</code></pre>\n\
  </div>\n\
\n\
    <div class=\"related-links\">\n\
  \n\
    <div class=\"familylinks\">\n\
      <div class=\"parentLink\">\n\
        <strong>Parent page:</strong> <a class=\"link\" href=\"../Scripted-Right-Click-Menus.html\">Scripted Right-Click Menus</a>\n\
      </div>\n\
  \n\
    </div>\n\
  \n\
      <div class=\"relinfo\">\n\
        <h4 class=\"related\">Related pages</h4>\n\
        <ul>\n\
          <li><a class=\"link\" href=\"../Scripted-Right-Click-Menus.html\">Scripted Right-Click Menus</a></li>\n\
          <li><a class=\"link\" href=\"RCMenu-User-Interface-Items.html\">RCMenu User-Interface Items</a></li>\n\
        </ul>\n\
      </div>\n\
    </div>\n\
\n\
</body>\n\
</html>\n\
";